/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of one of the parties involved in the TRANSORG project. For
 * 	  written permission, please contact diasz@ida.liu.se.
 *
 * 4. Products derived from this Software may not be called "TRANSORG"
 *    nor may "TRANSORG" appear in their names without prior written
 *    permission.
 *
 * 5. Due credit should be given to the TRANSORG Project
 *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
 *
 * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
 *
 * $Id: AckReadMessage.java,v 1.0   Exp $
 *
 */

package universal;

/*
 * Author		: Rahul Agarwal
 * Modify      : Isabelle Ravot
 * Date         : 01/08/02
 * Last changes : 20/09/02
 * Contact		: diasz@ida.liu.se
 */

import java.io.*;
import java.util.*;

/**
 *<dt>Used for creating AckReadMessage.
 *<dt>An AckReadMessage is sent during the consensus by all the replicated server
 *<dt>to the leader to acknowledge the Read message that the leader has sent.
 *<dt>It contains the last value (Request, Reply, Update) reading and the last round number it has written this value for this place in the total order.
 */
public class AckReadMessage extends ReadMessage{

    /** The last value read for the place in the total order for which it has received the read message */
    public Outcome value;

    /** The last round number it has written for the place in the total order for which it has received the read message */
    public int lastwrite;

    /**
     *The constructor
     *@param batchno the place in the total order for which the consensus is actually made
     *@param roundno the round number for this consensus
     *@param lastwrite the last round number the sender of this message  has written for the place in the
     *total order
     *@param value the last value read at the round number lastwrite
     */
    public AckReadMessage(int batchno, int roundno, int lastwrite, Outcome value)
    {

	super(batchno, roundno);
	msgType = ACK_READ_MSG;

	this.value = value;
	this.lastwrite = lastwrite;

	/*
	  Debug.println("I am trying to send a ackread message but :"+value+"for the batch"+batchno+"for the roundno"+roundno);
	*/
    }

    /**
     *If you want to decode an AckReadMessage, you should call the method Decode of
     *the superclass Message, not this one.
     *@return an AckReadMessage but as a type of ReadMessage(superclass)
     */
    public static ReadMessage decode(DataInputStream dis)
    {
	int newBatchno;
	int newRoundno;
	int newLastWrite;
	Outcome newValue;

	try
	    {
		/* encoding:

		   dos.writeInt(msgType);
		   dos.writeInt(senderid);
		   dos.writeInt(mid);
		   dos.writeInt(batchno);
		   dos.writeInt(roundno);
		   dos.writeInt(lastwrite);
		   if (value != null)
		   {
		   dos.writeInt(value.size());
		   for (int i=0; i<value.size(); i++)
		   {
		   TOBroadcastMessage m = (TOBroadcastMessage)value.get(i);
		   dos.write(m.encode());
		   }
		   }
		   else
		   {
		   dos.writeInt(-1);
		   }
		*/

		newBatchno = dis.readInt();
		newRoundno = dis.readInt();
		newLastWrite = dis.readInt();

		//int len = dis.readInt();
		//newValue = new Vector();
		/*
		  if (len != -1)
		  {
		  for (int i=0; i<len; i++)
		  {
		  newValue.addElement(UniversalMessage.fullDecode(dis));
		  }
		  }
		  else
		  newValue = null;
		*/
		boolean valHere = dis.readBoolean();
		if(valHere)
		    {
			newValue=new Outcome();
			newValue.readObject(dis);
		    }
		else
		    {
			newValue=null;
		    }

		/*
		  try
		  {
		  ObjectInputStream obj = new ObjectInputStream(dis);
		  newValue=(Outcome)obj.readObject();
		  obj.close();

		  }
		  catch (java.lang.ClassNotFoundException e)
		  {
		  e.printStackTrace();
		  }
		*/
		return new AckReadMessage(newBatchno, newRoundno, newLastWrite, newValue);
	    }
	catch (IOException e)
	    {
		e.printStackTrace();
	    }

	return null;
    }

    /**
     *If you want to encode a ReadMessage, you should call this method not the one from the
     *superclass.
     *@return an array of byte which contains all the information of this AckReadMessage
     */
    public byte[] encode()
    {
	ByteArrayOutputStream baos = new ByteArrayOutputStream();
	DataOutputStream dos = new DataOutputStream(baos);
	byte[] byteValue=null;
	try
	{
	    dos.writeInt(msgType);
	    dos.writeInt(senderid);
	    dos.writeInt(mid);
	    dos.writeInt(batchno);
	    dos.writeInt(roundno);
	    dos.writeInt(lastwrite);
	    if(value!=null)
		{
		    dos.writeBoolean(true);
		    value.writeObject(dos);
		}
	    else
		{
		    dos.writeBoolean(false);
		}

	    /*
	      ObjectOutputStream obj = new ObjectOutputStream(dos);
	      obj.writeObject(value);
	      obj.close();
	    */
	    /*
	      if (value != null)
	      {
	      dos.writeInt(value.size());
	      for (int i=0; i<value.size(); i++)
	      {
	      UniversalMessage m = (UniversalMessage)value.elementAt(i);
	      dos.write(m.encode());
	      }
	      }
	      else
	      {
	      dos.writeInt(-1);
	      }
	    */
	    dos.flush();
	    byteValue=baos.toByteArray();
	    baos.flush();
	    dos.close();
	    baos.close();
	}
	catch (IOException e)
	{
	    e.printStackTrace();
	}

	return byteValue;
    }


    /**
     * Compares two objects for equality. Returns a boolean that indicates
     * whether this object is equivalent to the specified object. This method
     * is used when an object is stored in a hashtable.
     * @param obj the Object to compare with
     * @return true if these Objects are equal; false otherwise.
     * @see java.util.Hashtable
     */
    public boolean equals(Object obj) {

	if (!(obj instanceof AckReadMessage))
	    return false;

	AckReadMessage m = (AckReadMessage)obj;

	if (m.senderid == senderid && m.batchno == batchno && m.roundno == roundno)
	    return true;
	else
	    return false;
    }



    public String toString() {
	return "ACK READ ("+batchno+", "+roundno+")";
    }
}
