/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of one of the parties involved in the TRANSORG project. For
 * 	  written permission, please contact diasz@ida.liu.se.
 *
 * 4. Products derived from this Software may not be called "TRANSORG"
 *    nor may "TRANSORG" appear in their names without prior written
 *    permission.
 *
 * 5. Due credit should be given to the TRANSORG Project
 *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
 *
 * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
 *
 * $Id: AckWriteMessage.java,v 1.0   Exp $
 *
 */

package universal;


/*
 * Author       : Rahul Agarwal
 * Modify       : Isabelle Ravot
 * Date         : 18/05/00
 * Last changes : 01/08/02
 * Contact		: diasz@ida.liu.se
 */

import java.io.*;

/**
 *<dt>Used for creating an AckWriteMessage.
 *<dt>An AckWriteMessage is sent during the consensus by all the replicated server
 *<dt>to the leader to acknowledge the WriteMessage that the leader has sent.
 *<dt>
 */

public class AckWriteMessage extends WriteMessage{

    /**the variable permission is not useful in a case of the crashStop model but could be used in the crashRecovery.*/
	public boolean permission;
    /**
     *To create an AckWriteMessage
     *@param batchno the place in the total order for which the consensus is actually made
     *@param roundno the round number for this consensus
     *@param permission not use in the crashStopModel
     */
    public AckWriteMessage(int batchno, int roundno, boolean permission)
    {
	msgType = ACK_WRITE_MSG;
	this.batchno = batchno;
	this.roundno = roundno;
	this.permission = permission;
    }

    /**
     *If you want to decode an AckWriteMessage, you should call the method Decode of
     *the superclass Message, not this one.
     *@return an AckWriteMessage but as a type of WriteMessage(superclass)
     */
    public static WriteMessage decode(DataInputStream dis)
    {
	int newBatchno;
	int newRoundno;
	boolean newPermission;

	try
	    {
		/* encoding:

		   dos.writeInt(msgType);
		   dos.writeInt(senderid);
		   dos.writeInt(mid);
		   dos.writeInt(batchno);
		   dos.writeInt(roundno);
		   dos.writeBoolean(permission);
		*/

		newBatchno = dis.readInt();
		newRoundno = dis.readInt();
		newPermission = dis.readBoolean();

		return new AckWriteMessage(newBatchno, newRoundno, newPermission);
	    }
	catch (IOException e)
	    {
		e.printStackTrace();
	    }

	return null;
    }
    /**
     *If you want to encode a WriteMessage, you should call this method not the one from the
     *superclass.
     *@return an array of byte which contains all the information of this AckWriteMessage
     */

    public byte[] encode()
    {
	ByteArrayOutputStream baos = new ByteArrayOutputStream();
	DataOutputStream dos = new DataOutputStream(baos);
	try
	{
	    dos.writeInt(msgType);
	    dos.writeInt(senderid);
	    dos.writeInt(mid);
	    dos.writeInt(batchno);
	    dos.writeInt(roundno);
	    dos.writeBoolean(permission);
	    dos.close();
	}
	catch (IOException e)
	{
	    e.printStackTrace();
	}

	return baos.toByteArray();
    }


    /**
     * Compares two objects for equality. Returns a boolean that indicates
     * whether this object is equivalent to the specified object. This method
     * is used when an object is stored in a hashtable.
     * @param obj the Object to compare with
     * @return true if these Objects are equal; false otherwise.
     * @see java.util.Hashtable
     */
    public boolean equals(Object obj) {

	if (!(obj instanceof AckWriteMessage))
	    return false;

	AckWriteMessage m = (AckWriteMessage)obj;

	if (m.senderid == senderid && m.batchno == batchno && m.roundno == roundno)
	    return true;
	else
	    return false;
    }


    public String toString()
    {
	return "ACK WRITE (" + batchno + ", " + roundno + ", "+ permission+")";
    }
}
