/**
  * Redistribution and use of this software and associated
  * documentation ("Software"), with or without modification, are
  * permitted provided that the following conditions are met:
  *
  * 1. Redistributions of source code must retain copyright statements
  *    and notices.  Redistributions must also contain a copy of this
  *    document.
  *
  * 2. Redistributions in binary form must reproduce the above
  *    copyright notice, this list of conditions and the following
  *    disclaimer in the documentation and/or other materials provided
  *    with the distribution.
  *
  * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
  *    products derived from this Software without prior written
  *    permission of one of the parties involved in the TRANSORG project. For
  * 	  written permission, please contact diasz@ida.liu.se.
  *
  * 4. Products derived from this Software may not be called "TRANSORG"
  *    nor may "TRANSORG" appear in their names without prior written
  *    permission.
  *
  * 5. Due credit should be given to the TRANSORG Project
  *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
  *
  * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  * PARTICULAR PURPOSE ARE DISCLAIMED.
  *
  * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
  * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
  * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
  * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
  * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
  *
  * $Id: AMAliveMessage.java,v 1.0   Exp $
  *
  */

package universal;

/*
 * Author		: Rahul Agarwal
 * Modify       : Isabelle Ravot
 * Date         : 01/08/02
 * Last changes : 01/08/02
 * Contact		: diasz@ida.liu.se
 */

import java.io.*;
import java.util.*;


/**
 * Used in leader election. Heartbeat message send periodically to inform the others replicas and
 * the client that the sender is still alive
 * Contact: diasz@ida.liu.se
 */
public class AmAliveMessage extends Message
{
    /**
     *the variable epochArray is not useful in the crashStop model
     *but could be in the crash recovery model
     */
    public int epochArray[];


    public AmAliveMessage(int epochArray[])
    {
	msgType = AM_ALIVE_MSG;

	this.epochArray = epochArray;
    }
    /**
     *If you want to decode an AmAliveMessage, you should call the method Decode of
     *the superclass Message, not this one.
     *@return an AmAliveMessage
     */
    public static AmAliveMessage decode(DataInputStream dis)
    {
	int newEpochArray[] = new int[Constants.MAX_PROCESSES];

	try
	    {
		/* encoding:

		   dos.writeInt(msgType);
		dos.writeInt(senderid);
		dos.writeInt(mid);
		if (epochArray != null)
		{
		dos.writeInt(epochArray.length);
		for (int i=0; i<epochArray.length; i++)
		{
		int epoch = epochArray[i];
		dos.writeInt(epoch);
		}
		}
		else
		{
			dos.writeInt(-1);
		}
		*/

		int len = dis.readInt();
		if (len != -1)
		    {
			for (int i=0; i<len; i++)
			    {
				newEpochArray[i] = dis.readInt();
			    }
		    }
		else
		    newEpochArray = null;

		return new AmAliveMessage(newEpochArray);
	    }
	catch (IOException e)
	    {
		e.printStackTrace();
	    }

	return null;
    }

     /**
     *If you want to encode an AmAliveMessage, you should call this method not the one from the
     *superclass.
     *@return an array of byte which contains all the information of this AmAliveMessage.
     */
    public byte[] encode()
    {
	ByteArrayOutputStream baos = new ByteArrayOutputStream();
	DataOutputStream dos = new DataOutputStream(baos);
	try
	{
	    dos.writeInt(msgType);
	    dos.writeInt(senderid);
	    dos.writeInt(mid);
	    if (epochArray != null)
		{
		    dos.writeInt(epochArray.length);
		    for (int i=0; i<epochArray.length; i++)
			{
			    int epoch = epochArray[i];
			    dos.writeInt(epoch);
			}
		}
	    else
		{
		    dos.writeInt(-1);
		}
	    dos.close();
	}
	catch (IOException e)
	{
	    e.printStackTrace();
	}

	return baos.toByteArray();
    }


    public String toString()
    {
	return "AM ALIVE";
    }
}
