/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of one of the parties involved in the TRANSORG project. For
 * 	  written permission, please contact diasz@ida.liu.se.
 *
 * 4. Products derived from this Software may not be called "TRANSORG"
 *    nor may "TRANSORG" appear in their names without prior written
 *    permission.
 *
 * 5. Due credit should be given to the TRANSORG Project
 *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
 *
 * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
 *
 * $Id: Message.java,v 1.0  diasz Exp $
 *
 * Date         Author  Changes
 */

package universal;

/*
 * Author       : Rahul Agarwal
 * Modify       : Isabelle Ravot
 * Date         : 25/05/00
 * Last changes : 01/08/02
 * Contact		: diasz@ida.liu.se
 */

import java.net.*;
import universal.communication.*;
import java.io.*;
import java.io.Serializable;

/**
 *This class is the superclass for all the kind of messages exchanges for the
 *consensus and leader election purposes.
 */

public abstract class Message implements Serializable{

  /** the pid of the replicated server that send this message */
  public int senderid = -1;

  // Unique message id for sender
  public int mid = -1;
    /**
     *Define the type of the message.
     *<dl>For example if it is an AckReadMessage or an AckWriteMessage,..
     */
  public int msgType;

    // public static final int ACK_FAST_WRITE_MSG = 0;
     public static final int ACK_READ_MSG = 1;
     public static final int ACK_WRITE_MSG = 2;
     public static final int AM_ALIVE_MSG = 3;
    // public static final int ASK_ALIVE_MSG = 4;
    // public static final int ASSIGNMENT_MSG = 5;
    // public static final int CORRECT_MSG = 6;
    // public static final int DECISION_MSG = 7;
    // public static final int FAST_WRITE_MSG = 8;
    // public static final int JOIN_MSG = 9;
    // public static final int NACK_FAST_WRITE_MSG = 10;
     public static final int NACK_READ_MSG = 11;
     public static final int NACK_WRITE_MSG = 12;
    // public static final int NEW_ENTRY_MSG = 13;
    // public static final int NEW_LEADER_MSG = 14;
     public static final int READ_MSG = 15;
    // public static final int RECOVERED_MSG = 16;
    // public static final int RECOVERING_MSG = 17;
    // public static final int UNIVERSAL_MSG = 18;
     public static final int WRITE_MSG = 19;

    // public static final int BATCH_MSG = 20;

    /**
     *When you want to decode a message whatever is its type you have to call
     *the Decode function of this class which is going to first decode the common variable
     *of all the messages and then to call the Decode function of his real type.
     */
public static Message decode(byte[] byteArray)
{
	int newMsgType;
	int newSenderid;
	int newMid;
	Message m = null;

	ByteArrayInputStream bais = new ByteArrayInputStream(byteArray);
	DataInputStream dis = new DataInputStream(bais);
	try
	{
		newMsgType = dis.readInt();
		newSenderid = dis.readInt();
		// jinsight
		newMid = dis.readInt();
		switch (newMsgType)
		{
		    	case ACK_READ_MSG :
				m = AckReadMessage.decode(dis);
				break;
			case ACK_WRITE_MSG :
				m = AckWriteMessage.decode(dis);
				break;

			case AM_ALIVE_MSG :
				m = AmAliveMessage.decode(dis);
				break;
		     /*
                        case ASK_ALIVE_MSG :
				m = AskAliveMessage.decode(dis);
				break;
			case ASSIGNMENT_MSG :
				m = AssignmentMessage.decode(dis);
				break;
			case CORRECT_MSG :
				m = CorrectMessage.decode(dis);
				break;
			case DECISION_MSG :
				m = DecisionMessage.decode(dis);
				break;
			case JOIN_MSG :
				m = JoinMessage.decode(dis);
				break;
		     */
		case NACK_READ_MSG :
		    m = NackReadMessage.decode(dis);
		    break;
		case NACK_WRITE_MSG :
		    m = NackWriteMessage.decode(dis);
		    break;
		    /*
			case NEW_ENTRY_MSG :
				m = NewEntryMessage.decode(dis);
				break;
		     */
		case READ_MSG :
		    m = ReadMessage.decode(dis);
		    break;
		    /*
			case RECOVERING_MSG :
				m = RecoveringMessage.decode(dis);
				break;
			case RECOVERED_MSG :
				m = RecoveredMessage.decode(dis);
				break;

			case UNIVERSAL_MSG :
				m = TOBroadcastMessage.decode(dis);
				break;
		     */
		case WRITE_MSG :
		    m = WriteMessage.decode(dis);
		    break;

		    /*
			case BATCH_MSG :
				m = MessageBatch.decode(dis);
				break;
		     */
		}
		if (m != null)
		{
			m.senderid = newSenderid;
			m.mid = newMid;
			return m;
		}

		dis.close();
		bais.close();
	}
	catch (IOException e)
	{
		e.printStackTrace();
	}
	return m;
}

    /**
     *When you want to encode a message, you should call the encode function of the real type
     *of the message and not this one.
     */
public byte[] encode()
{
	ByteArrayOutputStream baos = new ByteArrayOutputStream();
	DataOutputStream dos = new DataOutputStream(baos);
	try
	{
		dos.writeInt(msgType);
		dos.writeInt(senderid);
		dos.writeInt(mid);
	}
	catch (IOException e)
	{
		e.printStackTrace();
	}

	return baos.toByteArray();
}
	public int msgId(){
		return mid;
	}
	public int msgType(){
		return msgType;
	}
}
