/**
  * Redistribution and use of this software and associated
  * documentation ("Software"), with or without modification, are
  * permitted provided that the following conditions are met:
  *
  * 1. Redistributions of source code must retain copyright statements
  *    and notices.  Redistributions must also contain a copy of this
  *    document.
  *
  * 2. Redistributions in binary form must reproduce the above
  *    copyright notice, this list of conditions and the following
  *    disclaimer in the documentation and/or other materials provided
  *    with the distribution.
  *
  * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
  *    products derived from this Software without prior written
  *    permission of one of the parties involved in the TRANSORG project. For
  * 	  written permission, please contact diasz@ida.liu.se.
  *
  * 4. Products derived from this Software may not be called "TRANSORG"
  *    nor may "TRANSORG" appear in their names without prior written
  *    permission.
  *
  * 5. Due credit should be given to the TRANSORG Project
  *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
  *
  * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  * PARTICULAR PURPOSE ARE DISCLAIMED.
  *
  * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
  * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
  * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
  * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
  * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
  *
  * $Id: NackReadMessage.java,v 1.0  diasz Exp $
  *
  * Date         Author  Changes
  */

package universal;

/*
 * Author       : Rahul Agarwal
 * Modify       : Isabelle Ravot
 * Date         : 25/05/00
 * Last changes : 25/05/00
 * Contact		: diasz@ida.liu.se
 */

import java.io.*;
/**
 *<dt>Used for creating NackReadMessage.
 *<dt>A NackReadMessage is sent during the consensus by all the replicated server
 *<dt>to the leader to say that they don't acknowledge the Read message that the leader has sent.
 */
public class NackReadMessage extends ReadMessage
{
    /**
     *The constructor
     *@param batchno the place in the total order for which the consensus is actually made
     *@param roundno the round number for this consensus.
     */
    public NackReadMessage(int batchno, int roundno)
    {
	super(batchno, roundno);

	msgType = NACK_READ_MSG;
    }
    /**
     *When you want to decode a NackReadMessage you have to call the decode function
     *of the superclass Message.
     */
    public static ReadMessage decode(DataInputStream dis)
    {
	int newBatchno;
	int newRoundno;

	try
	    {
		/* encoding:

		   dos.writeInt(msgType);
		   dos.writeInt(senderid);
		   dos.writeInt(mid);
		   dos.writeInt(batchno);
		   dos.writeInt(roundno);
		*/

		newBatchno = dis.readInt();
		newRoundno = dis.readInt();

		return new NackReadMessage(newBatchno, newRoundno);
	    }
	catch (IOException e)
	    {
		e.printStackTrace();
	    }

	return null;
    }
    /**
     *When you want to encode a NackReadMessage you should call this function.
     */
    public byte[] encode()
    {
	ByteArrayOutputStream baos = new ByteArrayOutputStream();
	DataOutputStream dos = new DataOutputStream(baos);
	try
	{
	    dos.writeInt(msgType);
	    dos.writeInt(senderid);
	    dos.writeInt(mid);
	    dos.writeInt(batchno);
	    dos.writeInt(roundno);
	    dos.close();
	}
	catch (IOException e)
	{
	    e.printStackTrace();
	}

	return baos.toByteArray();
    }


    /**
     * Compares two objects for equality. Returns a boolean that indicates
     * whether this object is equivalent to the specified object. This method
     * is used when an object is stored in a hashtable.
     * @param obj the Object to compare with
     * @return true if these Objects are equal; false otherwise.
     * @see java.util.Hashtable
     */
    public boolean equals(Object obj) {

	if (!(obj instanceof NackReadMessage))
	    return false;

	NackReadMessage m = (NackReadMessage)obj;

	if (m.senderid == senderid && m.batchno == batchno && m.roundno == roundno)
	    return true;
	else
	    return false;
    }


    public String toString() {
	return "NACK READ ("+batchno+", "+roundno+")";
    }
}
