/**
  * Redistribution and use of this software and associated
  * documentation ("Software"), with or without modification, are
  * permitted provided that the following conditions are met:
  *
  * 1. Redistributions of source code must retain copyright statements
  *    and notices.  Redistributions must also contain a copy of this
  *    document.
  *
  * 2. Redistributions in binary form must reproduce the above
  *    copyright notice, this list of conditions and the following
  *    disclaimer in the documentation and/or other materials provided
  *    with the distribution.
  *
  * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
  *    products derived from this Software without prior written
  *    permission of one of the parties involved in the TRANSORG project. For
  * 	  written permission, please contact diasz@ida.liu.se.
  *
  * 4. Products derived from this Software may not be called "TRANSORG"
  *    nor may "TRANSORG" appear in their names without prior written
  *    permission.
  *
  * 5. Due credit should be given to the TRANSORG Project
  *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
  *
  * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  * PARTICULAR PURPOSE ARE DISCLAIMED.
  *
  * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
  * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
  * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
  * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
  * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
  *
  * $Id: OutcomeStore.java,v 1.0  diasz Exp $
  *
  * Date         Author  Changes
  */

package universal;
 /*
 * Author       : Isabelle Ravot
 * Date         : 01/08/02
 * Last changes : 01/08/02
 * Contact		: diasz@ida.liu.se
 */

import java.io.*;
import java.util.*;

/**
 * Used  to store the decision return by the consensus when it is sucessful.
 */
public class OutcomeStore
{
    /** The vector uses to store the decisions*/
    public Vector outcomes = new Vector();
    int i;
    // constructor at process pi
    public OutcomeStore()
    {
    }
    /**
     *Check whether a Request has already been processed.
     *<dl>It also checks if some requests have their time expires and if it is the case
     *that means that it's not possible that the client resends this request again, so it removes it.</dl>
     * @return true if the request has already been committed with the Outcome (Request,Reply,Update)
     *corresponding or false and null.
     */
    public synchronized StoreResult isCommitted(Request req)
    {
        StoreResult res;


	for (int i=0; i <outcomes.size();i++)
	    {

	      OutcomeReqRepl out=(OutcomeReqRepl)(outcomes.elementAt(i));

	      if((out.req.retention_id==req.retention_id)&&(out.req.client_id.compareTo(req.client_id))==0)
		  {


		      res=new StoreResult(true,out);
		      return res;

		  }
	    }

      	res = new StoreResult(false,null);

	//first Check is there is some outcome recorded wich are expired

	if(outcomes.size()>=universal.Constants.NUMBER_REQUEST)
	    {
		for(int i=0;i<outcomes.size();i++)
		    {
			OutcomeReqRepl out=(OutcomeReqRepl)(outcomes.elementAt(i));
			//Debug.println("the size of my vector is "+outcomes.size());
			if((out.req.expiration_time)<(System.currentTimeMillis()))
			    {
				outcomes.remove(out);
			    }
			//Debug.println("the size of my vector is after "+outcomes.size());
		    }

	    }


        return res;
    }
    /**
     * Stores the Outcome (Request, Reply, Update) return by the consensus when it is successful.
     */
    public synchronized void setCommitted(OutcomeReqRepl out)
    {

	outcomes.add(outcomes.size(),out);



    }
}
