/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of one of the parties involved in the TRANSORG project. For
 * 	  written permission, please contact diasz@ida.liu.se.
 *
 * 4. Products derived from this Software may not be called "TRANSORG"
 *    nor may "TRANSORG" appear in their names without prior written
 *    permission.
 *
 * 5. Due credit should be given to the TRANSORG Project
 *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
 *
 * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
 *
 * $Id: ReadMessage.java,v 1.0  diasz Exp $
 *
 * Date         Author  Changes
 */

package universal;


/*
 * Author       : Rahul Agarwal
 * Date         : 25/05/00
 * Last changes : 25/05/00
 * Contact		: diasz@ida.liu.se
 */

import java.io.*;
/**
 *<dt>Used for creating ReadMessage.
 *<dt>A ReadMessage is sent during the consensus by the leader to all the replicated servers
 *<dt>to say that it wants to choose a request for this position in the total order.
 *<dl>The replicated servers are going to answer by an AckReadMessage if they don't have already
 *read or write for a bigger round number. If they have already write for a smaller round number
 *the AckReadMessage contains the value that has already been written.</dl>
 *<dl>If they have read or write for a bigger round number they send a NackReadMessage.</dl>
 */
public class ReadMessage extends Message{
    /** roundno the round number for this consensus. */
	public int roundno;
    /**batchno the place in the total order for which the consensus is actually made */
	public int batchno;

    /**
     *@param batchno the place in the total order for which the consensus is actually made
     *@param roundno the round number for this consensus.
     */
	public ReadMessage(int batchno, int roundno)
	{
		msgType = READ_MSG;

		this.batchno = batchno;
		this.roundno = roundno;
	}
    /**
     *When you want to decode a ReadMessage you have to call the decode function
     *of the superclass Message.
     */

	public static ReadMessage decode(DataInputStream dis)
	{
		int newBatchno;
		int newRoundno;

		try
		{
			/* encoding:

			dos.writeInt(msgType);
			dos.writeInt(senderid);
			dos.writeInt(mid);
			dos.writeInt(batchno);
			dos.writeInt(roundno);
			*/

			newBatchno = dis.readInt();
			newRoundno = dis.readInt();

			return new ReadMessage(newBatchno, newRoundno);
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}

		return null;
	}

    /**
     *When you want to encode a ReadMessage you should call this function.
     */
	public byte[] encode()
	{
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DataOutputStream dos = new DataOutputStream(baos);
		try
		{
			dos.writeInt(msgType);
			dos.writeInt(senderid);
			dos.writeInt(mid);
			dos.writeInt(batchno);
			dos.writeInt(roundno);
			dos.close();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}

		return baos.toByteArray();
	}

	/**
	 * Compares two objects for equality. Returns a boolean that indicates
	 * whether this object is equivalent to the specified object. This method
	 * is used when an object is stored in a hashtable.
	 * @param obj the Object to compare with
	 * @return true if these Objects are equal; false otherwise.
	 * @see java.util.Hashtable
	 */
	public boolean equals(Object obj) {

		if (!(obj instanceof ReadMessage))
			return false;

		ReadMessage m = (ReadMessage)obj;

		if (m.senderid == senderid && m.batchno == batchno && m.roundno == roundno)
			return true;
		else
			return false;
	}

	public String toString()
	{
		return "READ ("+batchno+", "+roundno+")";
	}
}
