/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of one of the parties involved in the TRANSORG project. For
 * 	  written permission, please contact diasz@ida.liu.se.
 *
 * 4. Products derived from this Software may not be called "TRANSORG"
 *    nor may "TRANSORG" appear in their names without prior written
 *    permission.
 *
 * 5. Due credit should be given to the TRANSORG Project
 *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
 *
 * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
 *
 * $Id: Request.java,v 1.0  diasz Exp $
 *
 * Date         Author  Changes
 */

package universal;
/*
 * Author       : Isabelle Ravot
 * Date         : 01/08/02
 * Last changes : 01/08/02
 * Contact		: diasz@ida.liu.se
 */

import java.io.*;
import java.util.*;

/**
 * Object  Request contains the retention id of the request, the client id and its expirarion time.
 *<dl>The retention id and the client id together permit to identify uniquely the request.</dl>
 *<dl>The expiration time permit to remove the request from the OutcomeStore if the time is over.</dl>
 */

public class Request implements Serializable
{
    /**Each different request from a certain client has a different id*/
    public int retention_id;
    /**Each client has its own client id*/
    public String client_id;
    /**Useful to know when we can remove a request form the OutcomeStore*/
    public long expiration_time;

    public Request(int retention_id,String client_id,long expiration_time)
    {
	this.retention_id=retention_id;
	this.client_id=client_id;
	this.expiration_time=expiration_time;
    }
    public Request()
    {
    }

    public void writeObject(DataOutputStream dos)
    {
	try
	    {
		dos.writeInt(retention_id);
		dos.writeUTF(client_id);
		dos.writeLong(expiration_time);
	    }
	catch (IOException e)
	    {
		e.printStackTrace();
	    }
    }
    public void readObject(DataInputStream dis)
    {
	try
	    {
		retention_id=dis.readInt();
		client_id=dis.readUTF();
		expiration_time=dis.readLong();
	    }
	catch (IOException e)
	    {
		e.printStackTrace();
	    }
    }

}
