/**
  * Redistribution and use of this software and associated
  * documentation ("Software"), with or without modification, are
  * permitted provided that the following conditions are met:
  *
  * 1. Redistributions of source code must retain copyright statements
  *    and notices.  Redistributions must also contain a copy of this
  *    document.
  *
  * 2. Redistributions in binary form must reproduce the above
  *    copyright notice, this list of conditions and the following
  *    disclaimer in the documentation and/or other materials provided
  *    with the distribution.
  *
  * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
  *    products derived from this Software without prior written
  *    permission of one of the parties involved in the TRANSORG project. For
  * 	  written permission, please contact diasz@ida.liu.se.
  *
  * 4. Products derived from this Software may not be called "TRANSORG"
  *    nor may "TRANSORG" appear in their names without prior written
  *    permission.
  *
  * 5. Due credit should be given to the TRANSORG Project
  *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
  *
  * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  * PARTICULAR PURPOSE ARE DISCLAIMED.
  *
  * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
  * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
  * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
  * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
  * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
  *
  * $Id: Universal.java,v 1.0  diasz Exp $
  *
  * Date         Author  Changes
  */

package universal;
/*
 * Author       : Isabelle Ravot
 * Date         : 01/08/02
 * Last changes : 01/08/02
 * Contact		: diasz@ida.liu.se
 */


import java.util.*;
import java.io.*;
import java.net.*;
import universal.*;
import universal.leader.*;
import universal.communication.*;
import universal.messagein.*;
import universal.consensus.*;
import universal.consensus.register.*;

/**
 *<dl>
 *<dt>
 *This class is the superclass of the object UniversalCrashStop which is instantiate by the server.
 *<dt>
 *It can be later also used as a superclass for the crash recovery model.
 *This class starts the thread of the class readListener, writeListener, AckListener which waits for
 *messages for the consensus.
 *<dt>
 *</dl>
 *It finds a port free for the socket tcp to listen incomming message for the leader election and the consensus.
 *
 *It also instantiates the object Sender, Receiver and ReplicasAnnounce.
 */


public abstract class Universal
{
    // used for performance tests
    private Date startTime;

    public int totalDel = 0;

    /**
     *Object which treats the leader election
     */
    public Leader lead;
    /**
     *Object which waits for message comming to the socket
     */
    public Receiver receiver;
    /**
     *Object which send message through the socket
     */
    public Sender sender;
    /**
     *Sends Corba message to the other replicated servers to tell them in which port the TCP socket listen
     */
    public ReplicasAnnounce replicasAnnounce;



    /* For communication purposes */
    public final InetAddress add = getLocalInetAddress();
    InetAddress add1;
    public int port = 5000;
    public boolean waiting;
    //modser
    public ServerSocket serverSocket;

    //For consensus purposes
    //public Vector cons=new Vector();
    /**
     *Object for the consensus purposes
     */
    public Consensus cons;
    /**
     *Object for the consensus purposes
     */
    public Register reg;
    /**
     *The pid of this server
     */
    public int pid;
    /**
     *The name of the file which contains the name of the others replicated server and their pid
     */
    public String replicasNameFile;

    Thread t;

    /**
     *Each replicated servers has one object universal.
     *@param pid the pid of the replicated server.
     *@param replicasNameFile the name of the file which contains the list of the others replicated servers
     */
public Universal(int pid,String replicasNameFile)
{
    this.pid=pid;
    this.replicasNameFile = replicasNameFile;
    startTime = new Date();

}



private static InetAddress getLocalInetAddress(){
	try
	{
		return InetAddress.getLocalHost();
	}
	catch (UnknownHostException e)
	{
		e.printStackTrace();
		System.err.println("Unknown host exception in Universal static initializer");
		return null;
	}

}

    /**
     *Look for a port free to listen.
     *Instantiate the object receiver, sender, replicaAnnounce.
     *Call the method Announce of the object replicasAnnounce which is going to send some
     *corba message to inform the others replicas on which ports our socket listen.
     */
public void initializeUniversal()
{
    /* Initialization of variables */


    // Connecting socket
    boolean flag = true;
    try
	{
	    flag = false;
	    //modser
	    serverSocket = new ServerSocket(port);
	}
    catch (IOException e)
        {
	    flag = true;
            port++;

	}




	receiver = new Receiver(this);
	sender = new Sender(this);

        replicasAnnounce =new ReplicasAnnounce(this);
	replicasAnnounce.Announce();


}

public abstract void initializeTasks();

/**
 *Starts the thread ReadListenner, WriteListenner, AckListenner
 */
public void startTasks()
{
    reg.getReadlistener().start();
    reg.getWritelistener().start();
    reg.getAcklistener().start();
}


public String toString()
{
	if (sender != null)
		return "Process "+sender.pid;
	else
		return "Unassigned process";
}
}
