/**
  * Redistribution and use of this software and associated
  * documentation ("Software"), with or without modification, are
  * permitted provided that the following conditions are met:
  *
  * 1. Redistributions of source code must retain copyright statements
  *    and notices.  Redistributions must also contain a copy of this
  *    document.
  *
  * 2. Redistributions in binary form must reproduce the above
  *    copyright notice, this list of conditions and the following
  *    disclaimer in the documentation and/or other materials provided
  *    with the distribution.
  *
  * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
  *    products derived from this Software without prior written
  *    permission of one of the parties involved in the TRANSORG project. For
  * 	  written permission, please contact diasz@ida.liu.se.
  *
  * 4. Products derived from this Software may not be called "TRANSORG"
  *    nor may "TRANSORG" appear in their names without prior written
  *    permission.
  *
  * 5. Due credit should be given to the TRANSORG Project
  *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
  *
  * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  * PARTICULAR PURPOSE ARE DISCLAIMED.
  *
  * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
  * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
  * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
  * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
  * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
  *
  * $Id: UniversalClient.java,v 1.0  diasz Exp $
  *
  * Date         Author  Changes
  */

package universal;

/** Author		: Isabelle Ravot
  *	Last changed: 29/06/03
  * Contact		: diasz@ida.liu.se
  */

import java.util.*;
import java.io.*;
import java.net.*;
import universal.*;
import universal.leader.*;
import universal.communication.*;
import universal.messagein.*;
/**
 *This class is the superclass of the class UniversalClientCrashStop which is instantiated by the client application.
 *<dl>
 *It finds a port free for the socket tcp to listen incomming message for the leader election from the replicated servers.
 *</dl>
 *<dl>
 *It instantiates the object ReceiverClient and ClientAnnounce.
 */

public abstract class UniversalClient
{
    // used for performance tests
    private Date startTime;

    public int totalDel = 0;
    /**Object which waits for message comming to the socket*/
    public ReceiverClient receiverClient;
    /**Object which treats the leader election in the client side*/
    public LeaderCrashStopClient lead;
    /**Sends Corba message to the replicated servers to inform them on which port the client is listenning*/
    public ClientAnnounce clientAnnounce;



    /* For communication purposes */
    public final InetAddress add = getLocalInetAddress();
    InetAddress add1;
    public int port = 6000;
    public boolean waiting;
    public ServerSocket serverSocket;
    /**The name of the file which contains the list of the name of the host on which are the others replicated servers
     *and their associated pid.
     */
    public String serverNameFile;
    //solution provisoire


    Thread t;
    /**
     *@param The name of the file which contains the list of the name of the host on which are the others replicated servers
     *and their associated pid.
     */
public UniversalClient(String serverNameFile)
{
    this.serverNameFile=serverNameFile;
    startTime = new Date();

}



private static InetAddress getLocalInetAddress(){
	try
	{
		return InetAddress.getLocalHost();
	}
	catch (UnknownHostException e)
	{
		e.printStackTrace();
		System.err.println("Unknown host exception in Universal static initializer");
		return null;
	}

}
    /**
     *Look for a port free to listen.
     *Instantiate the object receiverClient,ClientAnnounce.
     *Call the method Announce of the object ClientAnnounce which is going to send some corba message
     *to inform the replicated servers on which port the client socket listen.
     */
public void initializeUniversal()
    {
	/* Initialization of variables */


	// Connecting socket
	boolean flag = true;
	while(flag==true)
	    {
		try
		    {
			flag = false;
			serverSocket = new ServerSocket(port);
		    }
		catch (IOException e)
		    {
			flag = true;
			port++;

		    }
	    }
	receiverClient = new ReceiverClient(this);


        clientAnnounce =new ClientAnnounce(this);
	clientAnnounce.Announce();


    }


public abstract void initializeTasks();


public void startTasks()
{

}



}

