/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of one of the parties involved in the TRANSORG project. For
 * 	  written permission, please contact diasz@ida.liu.se.
 *
 * 4. Products derived from this Software may not be called "TRANSORG"
 *    nor may "TRANSORG" appear in their names without prior written
 *    permission.
 *
 * 5. Due credit should be given to the TRANSORG Project
 *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
 *
 * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
 *
 * $Id: UniversalCrashStop.java,v 1.0  diasz Exp $
 *
 * Date         Author  Changes
 */

package universal;
/*
 * Author       : Isabelle Ravot
 * Date         : 01/08/02
 * Last changes : 01/08/02
 * Contact		: diasz@ida.liu.se
 */


import java.util.*;
import java.io.*;
import java.net.*;
import universal.*;


import universal.consensus.*;
import universal.consensus.register.*;

import universal.communication.*;
import universal.leader.*;
import universal.messagein.*;

/**
 *This subclass of Universal implements the crash stop model in the server side
 *<dt>
 *It is instantiated by the aplication server.
 *<dt>Crash Stop Model means once a server has crashed it never recovers
 *<dt>It instantiates the object LeaderCrashStopServer useful for the leader election part.
 *<dt>It also instantiates the object Register that is for the consensus purposes.
 */


public class UniversalCrashStop extends Universal
{

    /**
     *Each replicated servers instantiate one object UniversalCrashStop.
     *<dt>
     *<dt>call the constructor of its superclass
     *<dt>Call different method of its superclass universal in order to intialize
     *<dt>our infrastructure (instantiate some object, start some thread ).
     *@param pid the pid of the replicated server which instantiates it.
     *@param replicasNameFile the name of the file which contains the list
     *of all the others replicated server and their pid.
     *
     */
    public UniversalCrashStop(int pid,String replicasNameFile)
    {

	super(pid,replicasNameFile);

	initializeUniversal();
	initializeTasks();
	startTasks();

    }


    /**
     *Instantiate the object LeaderCrashStopserver (useful for the leader election)
     *Instantiate the object Register (useful for the consensus)
     */
    public void initializeTasks()
    {
	/* Will be needed later
	   reg = new Register(this);
	*/
	reg = new Register(this);
	lead = new LeaderCrashStopServer(this);


    }

}

