/**
  * Redistribution and use of this software and associated
  * documentation ("Software"), with or without modification, are
  * permitted provided that the following conditions are met:
  *
  * 1. Redistributions of source code must retain copyright statements
  *    and notices.  Redistributions must also contain a copy of this
  *    document.
  *
  * 2. Redistributions in binary form must reproduce the above
  *    copyright notice, this list of conditions and the following
  *    disclaimer in the documentation and/or other materials provided
  *    with the distribution.
  *
  * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
  *    products derived from this Software without prior written
  *    permission of one of the parties involved in the TRANSORG project. For
  * 	  written permission, please contact diasz@ida.liu.se.
  *
  * 4. Products derived from this Software may not be called "TRANSORG"
  *    nor may "TRANSORG" appear in their names without prior written
  *    permission.
  *
  * 5. Due credit should be given to the TRANSORG Project
  *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
  *
  * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  * PARTICULAR PURPOSE ARE DISCLAIMED.
  *
  * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
  * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
  * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
  * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
  * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
  *
  * $Id: ProcessDetails.java,v 1.0   Exp $
  *
  */

package universal.communication;
/*
 * Author       : Rahul Agarwal
 * Modify       : Isabelle Ravot
 * Date         : 25/05/00
 * Last changes : 01/08/02
 * Contact		: diasz@ida.liu.se
 */




import java.net.*;
import java.io.*;
import universal.*;
/**
 *Each replicated server and the client keeps a description of
 *all the replicated servers : the port where it is listenning, its IP
 *address, its pid.
 */

public class ProcessDetails implements Serializable
{
    /** The pid of the replicated server that is decribed by this object */
	public int pid;
    /**The IP address of the replicated server that is described by this object*/
	public InetAddress address;
    /**The port where this replicated server is listenning*/
	public int port;

	public ProcessDetails(int pid, InetAddress address, int port)
	{
		this.pid = pid;
		this.address = address;
		this.port = port;

	}

	public static ProcessDetails decode(DataInputStream dis)
	{
		int newPid;
		InetAddress newAddress;
		int newPort;

		try
		{
			/* encoding:

			dos.writeInt(pid);
			dos.writeUTF(address.toString());
			dos.writeInt(port);
			*/

			newPid = dis.readInt();
			newAddress = InetAddress.getByName(dis.readUTF());
			newPort = dis.readInt();

			return new ProcessDetails(newPid, newAddress, newPort);
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}

		return null;
	}

	public byte[] encode()
	{
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DataOutputStream dos = new DataOutputStream(baos);
		try
		{
			dos.writeInt(pid);
			dos.writeUTF(address.getHostAddress());
			dos.writeInt(port);
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}

		return baos.toByteArray();
	}

	public String toString()
	{
		return "Process "+pid+" at "+address+", port "+port;
	}
}
