/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of one of the parties involved in the TRANSORG project. For
 * 	  written permission, please contact diasz@ida.liu.se.
 *
 * 4. Products derived from this Software may not be called "TRANSORG"
 *    nor may "TRANSORG" appear in their names without prior written
 *    permission.
 *
 * 5. Due credit should be given to the TRANSORG Project
 *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
 *
 * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
 *
 * $Id: Receiver.java,v 1.0   Exp $
 *
 */

package universal.communication;
/*
 * Author		: Rahul Agarwal
 * Modify       : Isabelle Ravot
 * Date         : 25/05/00
 * Last changes : 01/08/02
 * Contact		: diasz@ida.liu.se
 */




import java.util.*;
import java.net.*;
import java.io.*;
import universal.*;
import universal.consensus.*;
import universal.consensus.register.*;
import universal.leader.*;

/**
 *
 *This subclass implements the crash stop model receiver. It waits for connection to the socket and then after
 * each time a connection has been accepted, it starts a new thread to wait
 *for the packet and treat the message.
 */
public class Receiver implements Runnable
{
    //useful in order to make the consensus
    /**Contains the Read messages that have been received but not yet analize in ReadListenner*/
    public Vector ReadBuffer;
    /**Contains the Write messages that have been recived but not yet analize in WriteListenner*/
    public Vector WriteBuffer;
    /**Contain the AckRead,NackRead,AckWrite and NackWrite messages that have been
     *received but not yet analize in AckListenner.
     */
    public Vector AckBuffer;

    /** The Universal object this receiver is linked to.*/
    Universal uni ;
    /**
     *It starts the thread that is going to listen for connection.
     *Initialize ReadBuffer, WriteBuffer and AckBuffer.
     *@param univ the universal object this receuiver is linked to.
     */
    public Receiver(Universal univ)
    {
	ReadBuffer = new Vector();
	WriteBuffer = new Vector();
	AckBuffer =new Vector();
	uni = univ;

	Thread t = new Thread(this, "Receiver");
	t.start();

    }
    /**@return lead the object leader of the object Universal*/
    public Leader leader() {
	return uni.lead;
    }

    /**
     *@return reg the object register of the object Universal
     */


    public Register reg()
    {
	return uni.reg;
    }


    /**
     * Waits for connection to the socket. After each time a connection has been accepted,
     *it starts a new thread to wait for the packet and treat the message.
     */

    public void run()
    {
	//TCP socket to imitate a relaible channel
	for(;;)
	    {
		Socket clientSocket = null;
		try
		    {
			//System.out.println("Before accepting the connexion");
			clientSocket = uni.serverSocket.accept();
			//System.out.println("After accept the connexion");
		    } catch (IOException e) {
			e.printStackTrace();
		    }
		new SocketReceiver(clientSocket,this).start();
	    }

    }
}
