/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of one of the parties involved in the TRANSORG project. For
 * 	  written permission, please contact diasz@ida.liu.se.
 *
 * 4. Products derived from this Software may not be called "TRANSORG"
 *    nor may "TRANSORG" appear in their names without prior written
 *    permission.
 *
 * 5. Due credit should be given to the TRANSORG Project
 *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
 *
 * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
 *
 * $Id: Sender.java,v 1.0   Exp $
 *
 */

package universal.communication;
/*
 * Author       : Rahul Agarwal
 * Modify		: Isabelle Ravot
 * Date         : 25/05/00
 * Last changes : 01/08/02
 * Contact		: diasz@ida.liu.se
 */


import java.util.*;
import java.net.*;
import java.io.*;
import universal.*;
import universal.leader.*;

/**
 *To send messages via the TCP socket (in order to simulate a reliable channel).
 *The messages send via the socket are those for the consensus and the leader election.
 */
public class Sender
{

    /**List of the replicated servers that messages for the consensus should be send*/
    public Vector processesReplicas;

    /**
     *List of the replicated server plus the client .
     *The AmAlive messages should be send to all of them.
     */
    public Vector processes;

    /**The pid of the process that is going to send messages*/
    public int pid;

    // public Vector xmitmsg;
    /**The Universal Object that this sender is linked to.*/
    public Universal uni;
    //public Socket socket=null;

    public Sender(Universal uni)
	{
	    this.uni = uni;
	    this.pid=uni.pid;
	    processesReplicas = new Vector();
	    processes = new Vector();

	}

    /**
     *Sends the message m through Universal's socket to the proccesses specified in
     *the list processesSend.
     *@param m The message that we want to send.
     *@param processesSend The list of processes this message should be send
     */
    public void s_send(Message m,Vector processesSend)
    {
	//Debug.println("I am sending a message to the other");
	Object psObject[];
	ProcessDetails ps[] = new ProcessDetails[Constants.MAX_PROCESSES];
	// psObject = processes.toArray();
	psObject = Debug.v2a(processesSend);
	for (int cou = 0; cou < psObject.length; cou++)
	    {
		ps[cou] = (ProcessDetails) psObject[cou];
		s_send(m, ps[cou].address, ps[cou].port);
	    }
    }


    /**
     * Send message through Universal's socket to the specified address and port
     */
    public void s_send(Message m, InetAddress address, int port)
    {
	//Debug.println("this is send message to"+address);
	m.senderid = pid;
	send(m, address, port);
    }


     /**
     *Sends the message m through Universal's socket to the proccesses specified in
     *the list processesSend.
     *@param m The message that we want to send.
     *@param processesSend The list of processes this message should be send
     */
    public void send(Message m, Vector processesSend)
    {
	Object psObject[];
	ProcessDetails ps[] = new ProcessDetails[Constants.MAX_PROCESSES];
	// psObject = processes.toArray();
	psObject = Debug.v2a(processesSend);
	for (int cou = 0; cou < psObject.length; cou++)
	    {
		ps[cou] = (ProcessDetails) psObject[cou];
		Debug.println("I send a message to"+ps[cou].address);
		send(m, ps[cou].address, ps[cou].port);
	    }

    }


    /**
     * Send message through Universal's socket to the specified address and port
     */
    public void send(Message m, InetAddress address, int port)
    {

	m.senderid = pid;



		OutputStream out=null;
		BufferedOutputStream bout=null;
		//DataOutputStream dout=null;
		Socket socket =null;
		byte[] byteArray = m.encode();
		try {
		    Debug.println(" sending " + m +" to "+address+": "+port+" the size of the array is "+byteArray.length);
		    //System.out.print(" send 1 ");
                     socket = new Socket(address, port);
		     //System.out.print(" send 2 ");
                     out = socket.getOutputStream();
		     bout=new BufferedOutputStream(out);
		     bout.write(byteArray,0,byteArray.length);
		     //dout=new DataOutputStream(out);
		     //System.out.print(" send 3 ");
		     /*
		       dout.writeInt(byteArray.length);
		       //dout.flush();
		       int fragment=0;
		       while(fragment<=byteArray.length)
		       {
		       if((fragment+1000)<=byteArray.length)
		       {

		       dout.write(byteArray,fragment,1000);
		       }
		       else
		       {

		       dout.write(byteArray,fragment,(byteArray.length-fragment));
		       }
		       fragment=fragment+1000;

		       }
		     */
		     bout.flush();
		     //dout.flush();
		     out.flush();
		     //System.out.println(" send 4 ");
		    /*
		      out.close();
		      System.out.print(" send 5 ");
		      socket.close();
		      System.out.println(" send 6 ");
		    */

                 } catch (UnknownHostException e) {
                     System.err.println("Don't know about host:"+address);

                 } catch (IOException e) {
                     Debug.println("Couldn't get I/O for "
                                        + "the connection to:"+address);
		     //e.printStackTrace();

                 }
		 finally{
		     try
			 {
			     if(bout!=null)
				 {
				     bout.close();
				 }
			     if(out!=null)
				 {

				     out.close();
				 }
			     if(socket!=null)
				 {
				     socket.close();
				 }
			 }
		     catch (IOException e) {
			 Debug.println("Couldn't close "
				       + "the connection to:"+address);

		     }

		 }
    }


    /**
     * Returns the number of replicated server in the list processesReplicas.
     */
    public int totalno()
    {
	return processesReplicas.size();
    }
}
