/**
  * Redistribution and use of this software and associated
  * documentation ("Software"), with or without modification, are
  * permitted provided that the following conditions are met:
  *
  * 1. Redistributions of source code must retain copyright statements
  *    and notices.  Redistributions must also contain a copy of this
  *    document.
  *
  * 2. Redistributions in binary form must reproduce the above
  *    copyright notice, this list of conditions and the following
  *    disclaimer in the documentation and/or other materials provided
  *    with the distribution.
  *
  * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
  *    products derived from this Software without prior written
  *    permission of one of the parties involved in the TRANSORG project. For
  * 	  written permission, please contact diasz@ida.liu.se.
  *
  * 4. Products derived from this Software may not be called "TRANSORG"
  *    nor may "TRANSORG" appear in their names without prior written
  *    permission.
  *
  * 5. Due credit should be given to the TRANSORG Project
  *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
  *
  * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  * PARTICULAR PURPOSE ARE DISCLAIMED.
  *
  * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
  * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
  * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
  * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
  * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
  *
  * $Id: SocketReceiver.java,v 1.0   Exp $
  *
  */

package universal.communication;
/*
 * Author       : Isabelle Ravot
 * Date         : 25/05/00
 * Last changes : 01/08/02
 * Contact		: diasz@ida.liu.se
 */




import java.util.*;
import java.net.*;
import java.io.*;
import universal.*;
import universal.consensus.*;
import universal.consensus.register.*;
import universal.leader.*;
import universal.utils.*;

/**
 *Wait for packet to arrive to the socket. Then depending on the type of the message,
 *it adds it to the vector AckBuffer, ReadBuffer or WriteBuffer if it is respectively an
 *acknowledgement, read or write messages.
 *If it is an Amalive message it inform the leader part of our algorithm that an AmAlive message has arrived.
 */

public class SocketReceiver extends Thread
{

    Socket clientSocket;
    /**The InputStream that arrive via the socket.*/
    InputStream in=null;
    Receiver rec;

    SocketReceiver(Socket clientSocket,Receiver rec)
    {
        this.clientSocket = clientSocket;
	this.rec=rec;
    }
    /**Depending on the type of the message,
     *it adds it to the vector AckBuffer, ReadBuffer or WriteBuffer if it is respectively an
     *acknowledgement, read or write messages.
     *If it is an Amalive message it inform the leader part of our algorithm that an AmAlive message has arrived. */
    public void run()
    {
	Message m = null;
	try
	    {
		Debug.println("Before receive");
		m = (Message) receive();
		Debug.println("after receive the type of the message was "+m.msgType);

	    }
	catch (ClassNotFoundException e)
	    {
		System.err.println("Class not found Exception in receiver ");
	    }
	catch (IOException e)
	    {
		//System.err.println("IO Exception in receiver ");
		e.printStackTrace();
		//continue;
	    }

	Debug.println(" receiving "+m+" from "+m.senderid);


	/* "switch" on type of message... */


	/* Messages for both types of Universal */

	// Will be useful when the consensus will be implemented

	if (m instanceof ReadMessage)
	    {
		ReadMessage rmsg=(ReadMessage)m;
		if((rmsg instanceof AckReadMessage)||(rmsg instanceof NackReadMessage))
		    {
			synchronized (rec.reg().getAcklistener())
			    {
				synchronized (rec.AckBuffer)
				    {
					if(m instanceof AckReadMessage)
					    {
						rec.AckBuffer.addElement((AckReadMessage) m);
					    }
					else
					    {
						rec.AckBuffer.addElement((NackReadMessage) m);
					    }
				//System.out.println("I add one element the size is now "+rec.AckBuffer.size());

				    }

				rec.reg().getAcklistener().notify();
				//System.out.println("notify acklistener");

			    }

		    }
		else
		    {
			//Debug.println("receive a read message from"+m.senderid+" and of type "+m.msgType);
			synchronized (rec.reg().getReadlistener())
			    {
				synchronized (rec.ReadBuffer)
				    {
					rec.ReadBuffer.addElement(rmsg);
				//System.out.println("I add one element the size is now "+rec.ReadBuffer.size());

				    }

				rec.reg().getReadlistener().notify();

			    }
		    }
	    }

	else

	    if (m instanceof WriteMessage)
		{
		    WriteMessage wmsg =(WriteMessage)m;
		    if((wmsg instanceof AckWriteMessage)||(wmsg instanceof NackWriteMessage))
			{
			    synchronized (rec.reg().getAcklistener())
				{
				    synchronized (rec.WriteBuffer)
					{
					    if(m instanceof AckWriteMessage)
						{
						    rec.AckBuffer.addElement((AckWriteMessage) m);
						}
					    else
						{
						    rec.AckBuffer.addElement((NackWriteMessage)m);
						}
					}
				    rec.reg().getAcklistener().notify();
				}
			}
		    else
			{
			    //Debug.println("receive a write message from"+m.senderid+" and of type "+m.msgType);
			    synchronized (rec.reg().getWritelistener())
				{
				    synchronized (rec.WriteBuffer)
					{
					    rec.WriteBuffer.addElement( wmsg);
					}
				    rec.reg().getWritelistener().notify();
				}
			}
		}
	    else
		if (m instanceof AmAliveMessage)
		    {
			if (rec.leader() != null)
			    rec.leader().receivedAmAlive((AmAliveMessage)m);
		    }


    }
    /**@return The message that has arrived as an Object*/
    public Object receive() throws IOException, ClassNotFoundException
    {

	//TCP socket to imitate a relaible channel
	byte[] byteArray =null;


	//System.out.print("receive1");

	byteArray=new byte[Constants.MAX_OBJECT_LENGTH];

	/* Modif. by diana on Feb. 19 2003, in order to cope with messages bigger than MAX_OBJECT_LENGTH*/

	java.util.Vector byteArrays=new java.util.Vector();

	//System.out.println("receive2");

	in = clientSocket.getInputStream();
	//System.out.println("receive3");
	//DataInputStream din = new DataInputStream(in);
	//System.out.println("receive4");
	BufferedInputStream bin=new BufferedInputStream(in);

	int length=0;
	int lengthread=0;
        int byteRead=0;
	int m=0, nr_arrays=0,i;
	ByteArrayBig crt;
	while(byteRead!=-1)
	    {
		byteRead=bin.read();
		if(m>=Constants.MAX_OBJECT_LENGTH){
		    byteArrays.addElement(new ByteArrayBig(m,byteArray,Constants.MAX_OBJECT_LENGTH));
		    byteArray=new byte[Constants.MAX_OBJECT_LENGTH];
		    m=0;
		    nr_arrays++;
		}
		byteArray[m]=(byte)byteRead;
		m=m+1;
	    }
	//System.out.println("m is "+m+" and nr_arrays is "+nr_arrays);
        if(m>0)
            byteArrays.addElement(new ByteArrayBig(m,byteArray,Constants.MAX_OBJECT_LENGTH));
        lengthread=nr_arrays*Constants.MAX_OBJECT_LENGTH+m;
	byte[] bigByteArray=new byte[lengthread],crt_array;
	int j,ba_cnt=0,crt_length;
	for(i=0;i<byteArrays.size();i++){
	    //byteArray=(ByteArrayBig)byteArrays.elementAt(i);
	    crt=(ByteArrayBig)byteArrays.elementAt(i);
            crt_length=crt.get_length();
            crt_array=crt.get_array();
	    for(j=0;j<crt_length;j++){
		bigByteArray[ba_cnt]=crt_array[j];
		ba_cnt++;
	    }
	}
	//	if(m==0)
	  /*
	  try
	  {
	  //System.out.println("receive5");
	  length=din.readInt();
	  //System.out.println("receive6");
	  byteArray = new byte[length];
	  //System.out.println("receive7");
	  while(lengthread!=length)
	  {
	  if((length-lengthread)<=1000)
	  {
	  lengthread = lengthread+din.read(byteArray,lengthread,(length-lengthread));
	  }
	  else
	  {
	  lengthread = lengthread+din.read(byteArray,lengthread,1000);
	  }

	  }
	  //din.readFully(byteArray);
	  }
	  catch(java.lang.Exception e)
	  {
	  System.out.println("problem with reading");
	  e.printStackTrace();
	  }
	  if(lengthread!=length)
	  {
	  System.out.println("I am completly desesperate");
	  }
	*/
	Debug.println("The size we really read "+lengthread);
	//	System.out.println("The size  we read "+length);

	 try
	     {
		 //din.close();
		 bin.close();
		 in.close();
		 clientSocket.close();
	     }
	 catch(java.io.IOException ex)
	     {
		 ex.printStackTrace();
	     }
	//in.close();
	//clientSocket.close();
	//UDP socket
	//DatagramPacket packet = new DatagramPacket(byteArray, byteArray.length);

	//uni.socket.receive(packet);

	return Message.decode(bigByteArray);

    }




}
