/**
  * Redistribution and use of this software and associated
  * documentation ("Software"), with or without modification, are
  * permitted provided that the following conditions are met:
  *
  * 1. Redistributions of source code must retain copyright statements
  *    and notices.  Redistributions must also contain a copy of this
  *    document.
  *
  * 2. Redistributions in binary form must reproduce the above
  *    copyright notice, this list of conditions and the following
  *    disclaimer in the documentation and/or other materials provided
  *    with the distribution.
  *
  * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
  *    products derived from this Software without prior written
  *    permission of one of the parties involved in the TRANSORG project. For
  * 	  written permission, please contact diasz@ida.liu.se.
  *
  * 4. Products derived from this Software may not be called "TRANSORG"
  *    nor may "TRANSORG" appear in their names without prior written
  *    permission.
  *
  * 5. Due credit should be given to the TRANSORG Project
  *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
  *
  * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  * PARTICULAR PURPOSE ARE DISCLAIMED.
  *
  * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
  * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
  * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
  * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
  * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
  *
  * $Id: Consensus.java,v 1.0   Exp $
  *
  */

package universal.consensus;
/*
 * Author		: Rahul Agarwal
 * Modify       : Isabelle Ravot
 * Date         : 01/08/02
 * Last changes : 01/08/02
 * Contact		: diasz@ida.liu.se
 */

import universal.*;
import universal.consensus.register.*;
import java.util.*;

/**
 *For each position in the total order the serverInterceptor instantiates a new Consensus.
 *And then for each roundno it calls the method propose of this class.
 */
public class Consensus {
    /**The object Universal the consensus is linked to*/
    public Universal univ;
    /**The position in the total order for which this object consensus has been created*/
    public int L;
    /**The same than L*/
    public Integer Li;
    /**The object in which we are going to register the Outcome value (Request,Reply,Update)
     *that has been decided for this position in the total order and also the last time it has
     *read and write for this position.
     */
    public RegistryReqRepl registry;

    public Consensus()
    {
    }
    /**
     *Initialize the value L and uni.
     *Create the object Registry for this position if it doesn't already exists.
     *It could exist already if an other leader has already processed a request for this position
     *which meam that we have already store something for this position in the total order.
     */
    public Consensus(Universal uni, int L)
    {

	univ = uni;
	this.L = L;
	Li=new Integer(L);
	if (!(univ.reg.result_registry).containsKey(Li))
	    {
		registry=new RegistryReqRepl(uni);
		(univ.reg.result_registry).put(Li,registry);
	    }
	registry = (RegistryReqRepl)((univ.reg.result_registry).get(Li));
    }
    /**
     *The leader propose the request, reply an update that it thinks should be for
     *this position in the total order.
     *The return of this method is either the value propose by the leader if no leader has
     *already processed a request for this postion in the total order or the value that has already been
     *decided for this position in the total order.
     */
    public Decision propose(int l, Outcome prop)
    {

	synchronized(univ.reg.getAcklistener())
	    {
		univ.reg.getAcklistener().initialize(l,L);
	    }
	Decision decision=new Decision();
	long startTime = System.currentTimeMillis();
	Debug.println("Consensus "+L+", propose "+l+" starting");

	ReadResult rr  = registry.read(l, L);
	if (rr.bool == true)
	    {
		Outcome o;

		if (rr.out == null)
		{
		    o = prop;
		    Debug.println("The value that we want to insert "+o);
		    Debug.println("Did read consensus "+L+" (read value is null)");
		}
		else
		    {
			o = rr.out;
			Debug.println("Did read consensus "+L+" (read value is not null)");
		}

		WriteResult wr = registry.write(l, o, L);
		if (wr.bool == true)
		    {
			Debug.println("Consensus "+L+", propose "+l+" successful in "+(System.currentTimeMillis()-startTime)+"ms");
			decision.deci=true;
			decision.out=o;
			System.out.println("The request for this batch "+L+" is "+decision.out.req.retention_id);
			System.out.println("The state for this batch "+L+" is size "+decision.out.upd_or_state.upd_or_state.length);


			return decision;
		    }
	    }
	Debug.println("Write/read consensus "+L+" unsuccessful");
	decision.deci=false;
	decision.out=null;
	return decision;
    }
}
