/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of one of the parties involved in the TRANSORG project. For
 * 	  written permission, please contact diasz@ida.liu.se.
 *
 * 4. Products derived from this Software may not be called "TRANSORG"
 *    nor may "TRANSORG" appear in their names without prior written
 *    permission.
 *
 * 5. Due credit should be given to the TRANSORG Project
 *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
 *
 * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
 *
 * $Id: AckListener.java,v 1.0   Exp $
 *
 */

package universal.consensus.register;

/*
 * Author       : Isabelle Ravot
 * Date         : 18/05/00
 * Last changes : 01/08/02
 * Contact		: diasz@ida.liu.se
 */

import universal.*;
import universal.communication.*;
import java.util.*;
import java.io.*;

/**Thread that wait for AckReadMessage, NackReadMessage, AckWriteMessage, NackwriteMessage to arrive.*/
public class AckListener extends Thread{
    /**The round number for which we are actually doing the consensus*/
    public int roundno;
    /**The position in the total order for which we are actually doing the consensus*/
    public int batchno;
    /**True if we have received  a majority + 1 AckRead or NackRead messages for this round number*/
    public boolean readrecvd;
    /**True if we have received a majority + 1  AckWrite or NackWrite for this round number */
    public boolean writerecvd;
    /**Number of Ackread messages we have received*/
    public int ackreadcount;
    /**Number of NackRead messages we have received*/
    public int nackreadcount;
    /**Number of AckWrite messages we have received*/
    public int ackwritecount;
    /**Number of NackWrite messages we have received*/
    public int nackwritecount;
    /**Could be useful if we want to implement the crash recovery model as it was done in Paxos*/
    public int falsepermissioncount;
    /**Number of AckRead+NackRead messages we have received*/
    public int readcount;
    /**Number of AckWrite+NackWrite messages we have received*/
    public int writecount;
    /**
     *If a request has already been processed for this position in the
     *total order,the AckRead messages contains the value decided and for for which round number it has been written.
     *In the variable remenber_highest_k_value we store the highest round number when the value has been
     *written.I
     */
    public int remember_highest_k_value;
     /**
     *If a request has already been processed for this position in the
     *total order,the AckRead messages contain the value decided and for for which round number it has been written.
     *In the variable remenber_highest_k_out we store the value that has been written for the highest round number.
     */
    public Outcome remember_highest_k_out;

    /**Could be useful if we want to implement the crash recovery model as it was done in Paxos*/
    private String filename;
    /**The object Universal this AckListenner is linked to*/
    protected Universal uni;


    Register reg;

    public AckListener(Register regi)
    {
	super("Acknowledgement Listener");
	reg = regi;
    }
    /**Initialize all the global variable*/
    public void initialize(int round,int batch)
    {
	roundno=round;
	batchno=batch;
	remember_highest_k_value=0;
	remember_highest_k_out=null;
	readrecvd= false;
	writerecvd = false;

	ackreadcount= 0;
	nackreadcount = 0;
	readcount = -1;

	ackwritecount = 0;
	nackwritecount = 0;
	writecount = -1;

	falsepermissioncount = 0;

    }
    /**
     *The thread that is going to wait for Acknowledgement message to arrive.
     *As soon as a majority+1 of replicated servers as acknowledge it notifies the
     *Registry.
     */
    public synchronized void ackListener()
    {
	Message m;

	try
	    {
		if (receiver().AckBuffer == null)
		    {
			//System.out.println("wait 1");
			this.wait();
		    }
		else
		    {
			if (Debug.v2a(receiver().AckBuffer).length == 0)
			    {
				//System.out.println("Receiver in ReadListener "+receiver());
				//System.out.println("wait 2");
				this.wait();
				//System.out.println("After wait 2 the size of ReadBuffer "+receiver().ReadBuffer.size());
			    }
		    }
	    }
	catch (InterruptedException e)
	    {
		e.printStackTrace();
		System.err.println("Interrupted Exception in read listener ");
	    }


	Object ackObject[] = Debug.v2a(receiver().AckBuffer);
	m = (Message)ackObject[0];
	synchronized (receiver().AckBuffer)
	    {
		System.out.println("The size of ackbuffer is "+receiver().AckBuffer.size()+"I remove one element");
		receiver().AckBuffer.removeElement(ackObject[0]);
	    }
	synchronized(this)
	    {
		if(m instanceof ReadMessage)
		    {
			ReadMessage rmsg = (ReadMessage)m;
			int K = rmsg.batchno;
			int k = rmsg.roundno;

			Integer Ki=new Integer(K);
			Integer ki=new Integer(k);
			//Debug.println("batchno"+m.batchno);
			//Debug.println("roundno"+m.roundno);


			if((k==roundno)&&(K==batchno))
			    {
				if (readcount == -1)
				    readcount = 1;
				else
				    readcount ++;

				// if a majority of ack/nacks haven't been received for this round yet
				if (readrecvd == false)
				    {
					if (rmsg instanceof AckReadMessage)
					    {
						AckReadMessage ackmsg = (AckReadMessage) rmsg;
						int lastwrite=ackmsg.lastwrite;
						if (lastwrite > remember_highest_k_value)
						    {
							remember_highest_k_value =ackmsg.lastwrite;
							Debug.println("I am writing in the registry for the value "+k+" with remenber Highest k value"+ackmsg.value );
							//modify
							remember_highest_k_out = ackmsg.value;
							ackreadcount ++;
						    }
					    }
					else
					    {
						nackreadcount ++;
					    }

					int maj = (int)Math.ceil((float)(sender().totalno() + 1) / (float)2);
					if (readcount < maj)
					    {
						Debug.println("Read batch "+K+", round "+k+" received "+readcount+" ack/nack read out of "+maj);
					    }
					else
					    {
						readrecvd = true;
						synchronized (((RegistryReqRepl)(reg.result_registry.get(Ki))))
						    {
							//System.out.println("NOTIFY in REGISTRYYYYYYYYYYYYYYYYYYYYYYYYYYYYY");
							((RegistryReqRepl)(reg.result_registry.get(Ki))).notify();
						    }
					    }
				    }
			    }
		    }
		else
		    {

			WriteMessage wmsg = (WriteMessage)m;
			int K = wmsg.batchno;
			int k = wmsg.roundno;
			Integer Ki=new Integer(K);
			Integer ki=new Integer(k);

			if((k==roundno)&&(K==batchno))
			    {

				if (writecount == -1)
				    writecount = 1;
				else
				    writecount ++;
				if (writerecvd == false)
				    {
					if (wmsg instanceof AckWriteMessage)
					    {
						ackwritecount ++;
						Debug.println("Take in consideration this Ackwrite message");
						if (!((AckWriteMessage)wmsg).permission)
						    falsepermissioncount ++;
					    }
					else
					    {
						nackwritecount ++;
					    }

					int maj = (int)Math.ceil((float)(sender().totalno() + 1) / (float)2);
					if (writecount < maj)
					    {
						Debug.println("Write batch "+K+", round "+k+" received "+writecount+" ack/nack write out of "+maj);
					    }
					else
					    {
						writerecvd = true;
						synchronized (((RegistryReqRepl)(reg.result_registry.get(Ki))))
						    {
							((RegistryReqRepl)(reg.result_registry.get(Ki))).notify();
						    }
					    }
				    }
			    }
		    }

	    }
    }


    public Receiver receiver() {
	return reg.uni.receiver;
    }

    public void run()
    {
	for (;;)
	    ackListener();
    }

    public Sender sender() {
	return reg.uni.sender;
    }
}
