/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of one of the parties involved in the TRANSORG project. For
 * 	  written permission, please contact diasz@ida.liu.se.
 *
 * 4. Products derived from this Software may not be called "TRANSORG"
 *    nor may "TRANSORG" appear in their names without prior written
 *    permission.
 *
 * 5. Due credit should be given to the TRANSORG Project
 *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
 *
 * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
 *
 * $Id: ReadListener.java,v 1.0   Exp $
 *
 */

package universal.consensus.register;

/*
 * Author       : Rahul Agarwal
 * Modify       : Isabelle Ravot
 * Date         : 18/05/00
 * Last changes : 01/08/02
 * Contact		: diasz@ida.liu.se
 */

import universal.*;
import universal.communication.*;
import java.util.*;
import java.io.*;

/**
 *Thread that waits for ReadMessage from the leader.
 */
public class ReadListener extends Thread{

	Register reg;

    public ReadListener(Register regi)
    {
	super("Read Listener");
	reg = regi;
    }
    /**
     *When a server receives a ReadMessage it sends an AckReadMessage or a NackReadMessage depending if
     *it has already read or written for an higer round number or not.
     */
    public synchronized void readListener()
    {
	//System.out.println("***********BEGIN READLISTENER-------------------------------------");

	ReadMessage m;
	RegistryReqRepl crt;
	boolean to_send_null=false;
	try
	    {
		if (receiver().ReadBuffer == null)
		    {
				//System.out.println("wait 1");
			this.wait();
		    }
		else
		    {
			if (Debug.v2a(receiver().ReadBuffer).length == 0)
			    {
				//System.out.println("Receiver in ReadListener "+receiver());
				//System.out.println("wait 2");
				this.wait();
				//System.out.println("After wait 2 the size of ReadBuffer "+receiver().ReadBuffer.size());
			    }
		    }
	    }
	catch (InterruptedException e)
	    {
		e.printStackTrace();
		System.err.println("Interrupted Exception in read listener ");
	    }


	Object readObject[] = Debug.v2a(receiver().ReadBuffer);
	m = (ReadMessage) readObject[0];
	synchronized (receiver().ReadBuffer)
	    {
		//System.out.println("The size of readbuffer is "+receiver().ReadBuffer.size()+"I remove one element");
		receiver().ReadBuffer.removeElement((ReadMessage) readObject[0]);
	    }
	int K = m.batchno;
	int k = m.roundno;

	Integer Ki=new Integer(K);
	Integer ki=new Integer(k);
	//Debug.println("batchno"+m.batchno);
	//Debug.println("roundno"+m.roundno);
	synchronized(reg.registry)
	    {

		if(! (reg.result_registry.containsKey(Ki))){
		    to_send_null=true;
		    reg.result_registry.put(Ki,new RegistryReqRepl(reg.uni));
		}
		/*	if (!(reg.registry.containsKey(Ki)))
			reg.registry.put(Ki,new Registry(reg.uni));

		    System.out.println("lastread "+((Registry)(reg.registry.get(Ki))).lastread+" lastwrite "+((Registry)(reg.registry.get(Ki))).lastwrite+" k "+k);*/
		crt=(RegistryReqRepl)(reg.result_registry.get(Ki));
		if (crt.lastwrite >= k || crt.lastread >= k)
		    {
			NackReadMessage nackreadmsg = new NackReadMessage(K, k);
			for(int i=0;i<sender().processesReplicas.size();i++)
			    {
				if(((ProcessDetails)(sender().processesReplicas.elementAt(i))).pid==m.senderid)
				    {
					ProcessDetails pj = (ProcessDetails) sender().processesReplicas.elementAt(i);
					sender().s_send(nackreadmsg, pj.address, pj.port);
				    }
			    }

		    }
		else
		    {
				// readi <- k
			crt.lastread = k;
			Outcome to_send;
			int after_req=-1;
			Outcome first_elem=null;
			if(reg.registry.size()>0){
			    first_elem=(Outcome)reg.registry.elementAt(0);
			    after_req=first_elem.after_n_req;
			}
			System.out.println("K is "+K+" and after_req is "+after_req);
			if(after_req>-1){
			    if(K<after_req)
				if(to_send_null) to_send=null;
				else
				    to_send=new Outcome(crt.out.req,crt.out.res,first_elem.upd_or_state,first_elem.is_state,first_elem.after_n_req);
			    else
				//if(K-after_req<reg.registry.size()){
				if(to_send_null)
				    to_send=null;
				else{
				    //System.out.println("(K-after_req) is "+(K-after_req));
				    Outcome elem_intres=universal.utils.Utils.elementAt(reg.registry,K);
				    if(elem_intres==null)
					to_send=null;
				    else
					/*if(crt.out==null)
					  System.out.println("crt.out is null");*/
					to_send=new Outcome(crt.out.req,crt.out.res,elem_intres.upd_or_state,elem_intres.is_state, elem_intres.after_n_req);
				}
			}
			else to_send=null;

			AckReadMessage ackreadmsg = new AckReadMessage(K, k, crt.lastwrite, to_send);
			for(int i=0;i<sender().processesReplicas.size();i++)
			    {
				if(((ProcessDetails)(sender().processesReplicas.elementAt(i))).pid==m.senderid)
				    {
					ProcessDetails pj = (ProcessDetails) sender().processesReplicas.elementAt(i);
					sender().s_send(ackreadmsg, pj.address, pj.port);
				    }
			    }
		    }
	    }
    }

    public Receiver receiver() {
	return reg.uni.receiver;
    }

    public void run()
    {
	for (;;)
	    readListener();
    }

    public Sender sender() {
	return reg.uni.sender;
    }
}
