/**
  * Redistribution and use of this software and associated
  * documentation ("Software"), with or without modification, are
  * permitted provided that the following conditions are met:
  *
  * 1. Redistributions of source code must retain copyright statements
  *    and notices.  Redistributions must also contain a copy of this
  *    document.
  *
  * 2. Redistributions in binary form must reproduce the above
  *    copyright notice, this list of conditions and the following
  *    disclaimer in the documentation and/or other materials provided
  *    with the distribution.
  *
  * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
  *    products derived from this Software without prior written
  *    permission of one of the parties involved in the TRANSORG project. For
  * 	  written permission, please contact diasz@ida.liu.se.
  *
  * 4. Products derived from this Software may not be called "TRANSORG"
  *    nor may "TRANSORG" appear in their names without prior written
  *    permission.
  *
  * 5. Due credit should be given to the TRANSORG Project
  *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
  *
  * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  * PARTICULAR PURPOSE ARE DISCLAIMED.
  *
  * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
  * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
  * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
  * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
  * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
  *
  * $Id: RegistryReqRepl.java,v 1.0   Exp $
  *
  */

package universal.consensus.register;

/*
 * Author       : Rahul Agarwal
 * Modify		: Isabelle Ravot and Diana Szentivayi (changed name from Registry to RegistryReqRepl)
 * Date         : 18/05/00
 * Last changes : 01/05/03
 * Contact		: diasz@ida.liu.se
 */

import universal.*;
import universal.communication.*;
import java.util.*;
import java.io.*;
/**
 *Each time a new consensus is started for a new position in the total order,
 *a registry is added to the hashtable of the register.
 *One registry contains the value that it has received in the WriteMessage from the leader for this position in the total order.
 *It also contains the last time this server has received a ReadMessage and  the last time it has received the WriteMessage.
 */
public class RegistryReqRepl{

    /**
     *The last time it has received a ReadMessage from the leader for the position in the total order
     *that this regitry has been instantiated.
     */
    public int lastread;
    /**
     *The last time it has received a WriteMessage from the leader for the position in the total order
     *that this regitry has been instantiated.
     */
    public int lastwrite;
    /**
     *The Outcome that was in the WriteMessage from the leader
     */
    public OutcomeReqRepl out ;


    public Universal uni;

    /**The object Universal that this Registry is linked to*/

    public RegistryReqRepl(Universal uni)
    {
	lastread = 0;
	lastwrite = 0;
	this.uni=uni;

    }

    /**
     *Read for the round number k and the position in the total order K.
     *It sends a ReadMessage and then wait for a majority + 1 of replicated server to acknowledge.
     *It returns an object ReadResult which as two variable. One boolean which is true if the
     *leader hasn't received NackReadMessage. One Outcome which is null if we have received one NackReadMessage
     *or if no request has already been processed for the position K in the total order. Otherwise it contains
     *the outcome (Request, Reply, Update) that has been decided for the position K.
     */
    public synchronized ReadResult read(int k, int K)
    {

	ReadResult rr = new ReadResult();
	ReadMessage msgread = new ReadMessage(K, k);

	sender().s_send(msgread,sender().processesReplicas);

	try
	    {
		if ((sender().totalno() != 1) /*&& (sender().totalno() != 0)*/)
		    {
			int maj = (int)Math.ceil((float)(sender().totalno() + 1) / (float)2);
			while (acklistener().readcount < maj)
			    {
				Debug.println("Registry "+K+" only received "+acklistener().readcount+" ack/nack read out of "+maj+", waiting...");
				this.wait();
			    }
		    }
	    }
	catch (InterruptedException e)
	    {
		e.printStackTrace();
		System.err.println("Interrupted Exception caught");
	    }

	if (acklistener().nackreadcount >= 1)
	    {
		//Debug.println(uni+" received nackRead, so decision false");
		rr.bool = false;
		return rr;
	    }
	rr.bool = true;


	rr.out = acklistener().remember_highest_k_out;
	//Debug.println(uni+" read decision true with out set: "+rr.out);
	return rr;
    }

    public Sender sender() {
	return uni.sender;
    }

    public AckListener acklistener()
    {
	return uni.reg.getAcklistener();
    }

    /*
      public void store(int k) throws java.io.IOException
      {
      ObjectOutputStream os = new ObjectOutputStream(new FileOutputStream(k + filename));
      os.writeInt(lastread);
      os.writeInt(lastwrite);
      os.writeObject(out);
      os.flush();
      os.close();
      }
    */
    /**
     *Try to write the value out for the round number k and the position in the total order K.
     *It sends a WriteMessage which contains the value out that it wants to write and then wait for a majority + 1
     * of replicated server to acknowledge.
     *It returns an object WriteResult which is a boolean that is true if the
     *leader hasn't received any NackWriteMessage.
     *WriteResult contains also an other variable but that is useful only if we want to implement the crash
     *recovery model as it was done in paxos.
     */
    public synchronized WriteResult write(int k, Outcome out, int K)
    {


	WriteMessage msgwrite = new WriteMessage(K, k, out);
	sender().s_send(msgwrite,sender().processesReplicas);

	try
	    {
		if ((sender().totalno() != 1) && (sender().totalno() != 0))
		    {
			int maj = (int)Math.ceil((float)(sender().totalno() + 1) / (float)2);
			while (acklistener().writecount < maj)
			    {
				Debug.println("Registry "+K+" only received "+acklistener().writecount+" ack/nack write, waiting...");
				this.wait();
			    }
		    }
	    }
	catch (InterruptedException e)
	    {
		e.printStackTrace();
	    }

	if (acklistener().nackwritecount >= 1)
	    {
		Debug.println(uni + " received nackWrite, so decision false");
		return new WriteResult(false, false);
	    }
	else
	    {
		if (acklistener().falsepermissioncount >= 1)
		    {
			Debug.println(uni + " received all ackWrites but false permission");
			return new WriteResult(true, false);
		    }
		else
		    {
			Debug.println(uni + " received all ackWrites and true permission");
			return new WriteResult(true, true);
		    }
	    }
    }
}
