/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of one of the parties involved in the TRANSORG project. For
 * 	  written permission, please contact diasz@ida.liu.se.
 *
 * 4. Products derived from this Software may not be called "TRANSORG"
 *    nor may "TRANSORG" appear in their names without prior written
 *    permission.
 *
 * 5. Due credit should be given to the TRANSORG Project
 *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
 *
 * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
 *
 * $Id: WriteListener.java,v 1.0   Exp $
 *
 */

package universal.consensus.register;

/*
 * @Author       : Rahul Agarwal
 * Modify		: Isabelle Ravot and Diana Szentivanyi
 * Date         : 18/05/00
 * Last changes : 23/02/03
 * Contact		: diasz@ida.liu.se
 *
 */

import universal.*;
import universal.communication.*;
import java.util.*;
import java.io.*;

/**
 *Thread that waits for WriteMessage from the leader.
 */
public class WriteListener extends Thread{

	Register reg;
    public WriteListener(Register regi)
    {
	super("Write Listener");
	reg = regi;
    }

    public Receiver receiver() {
	return reg.uni.receiver;
    }

    public void run()
    {
	for (;;)
	    writeListener();
    }

    public Sender sender() {
	return reg.uni.sender;
    }
    /**
     *When a server recceives a WriteMessage, it sends an AckWriteMessage or a NackWriteMessage
     *depending if it has already read or write for a higher round number or not.
     */
    public synchronized void writeListener()
    {

	WriteMessage m;
	RegistryReqRepl crt;
	int state_after_n;
	//	boolean to_send_null=false;
	try
	    {
		if (receiver().WriteBuffer == null)
		    {
			this.wait();
		    }
		else
		    if (Debug.v2a(receiver().WriteBuffer).length == 0)
			{
			    this.wait();
			}
	    }
	catch (InterruptedException e)
	    {
		Debug.println("Interrupted Exception in write listener ");
	    }
	synchronized(reg.registry)
	    {
		Object writeObject[];
		writeObject = Debug.v2a(receiver().WriteBuffer);

		if (writeObject.length != 0)
		    {
			m = (WriteMessage) writeObject[0];
			synchronized (receiver().WriteBuffer)
			    {
				receiver().WriteBuffer.removeElement((WriteMessage) writeObject[0]);
			    }

			int K = m.batchno;
			int k = m.roundno;
			Integer Ki=new Integer(K);
			Integer ki=new Integer(k);

			if(!reg.result_registry.containsKey(Ki)){
			    //to_send_null=true;
			    reg.result_registry.put(Ki, new RegistryReqRepl(reg.uni));
			}
			crt=(RegistryReqRepl)reg.result_registry.get(Ki);
			if(crt.lastwrite>k || crt.lastread>k){
				NackWriteMessage nackwritemsg = new NackWriteMessage(K, k);
				for(int i=0;i<sender().processesReplicas.size();i++)
				    {
					if(((ProcessDetails)(sender().processesReplicas.elementAt(i))).pid==m.senderid)
					    {
						ProcessDetails pj = (ProcessDetails) sender().processesReplicas.elementAt(i);
						sender().s_send(nackwritemsg, pj.address, pj.port);
					    }
				    }
			}
			else
			    {
				crt.lastwrite=m.roundno;
				crt.out=new OutcomeReqRepl(m.value.req,m.value.res);
				//do the pruning - modification done by Diana Szentianyi (diasz@ida.liu.se) - 17 feb. 2003

				if(m.value.is_state){
				    universal.utils.Utils.prune_outcome_vector(reg.registry,K);
				    reg.registry.insertElementAt(m.value,0);
				}
				else{
				    boolean add=true;
				    if(reg.registry.size()>0){
					Outcome first_elem=(Outcome)(reg.registry.elementAt(0));
					if(first_elem.after_n_req<K)
					    add=true;
					else
					    add=false;
				    }
				    if(add)
					reg.registry.addElement(m.value);
				}

				AckWriteMessage ackwritemsg;
				ackwritemsg = new AckWriteMessage(K, m.roundno, true);
				for(int i=0;i<sender().processesReplicas.size();i++)
				    {
					if(((ProcessDetails)(sender().processesReplicas.elementAt(i))).pid==m.senderid)
					    {
						ProcessDetails pj = (ProcessDetails) sender().processesReplicas.elementAt(i);
						sender().s_send(ackwritemsg, pj.address, pj.port);
					    }
				    }

			    }
		    }
	    }
    }
}


