/**
  * Redistribution and use of this software and associated
  * documentation ("Software"), with or without modification, are
  * permitted provided that the following conditions are met:
  *
  * 1. Redistributions of source code must retain copyright statements
  *    and notices.  Redistributions must also contain a copy of this
  *    document.
  *
  * 2. Redistributions in binary form must reproduce the above
  *    copyright notice, this list of conditions and the following
  *    disclaimer in the documentation and/or other materials provided
  *    with the distribution.
  *
  * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
  *    products derived from this Software without prior written
  *    permission of one of the parties involved in the TRANSORG project. For
  * 	  written permission, please contact diasz@ida.liu.se.
  *
  * 4. Products derived from this Software may not be called "TRANSORG"
  *    nor may "TRANSORG" appear in their names without prior written
  *    permission.
  *
  * 5. Due credit should be given to the TRANSORG Project
  *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
  *
  * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  * PARTICULAR PURPOSE ARE DISCLAIMED.
  *
  * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
  * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
  * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
  * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
  * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
  *
  * $Id: ClientAnnounce.java,v 1.0   Exp $
  *
  */

package universal.messagein;
/*
 * Author       : Isabelle Ravot
 * Date         : 01/08/02
 * Last changes : 01/08/02
 * Contact		: diasz@ida.liu.se
 */


import org.openorb.CORBA.*;
import org.omg.CORBA.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.net.*;
import universal.*;
import universal.communication.*;
import application.*;

/**
 *The client uses this class to send a CORBA message to all the replicated servers to
 *inform them in which port and which address they can send the message AmAlive needed
 *for the leader election.
 */
public class ClientAnnounce extends java.applet.Applet
{
    /**The UniversalClient this ClientAnnounce is linked to*/
   public UniversalClient uniClient;

    public ClientAnnounce(UniversalClient uniClient)
    {
	this.uniClient=uniClient;

    }
    /**
     *Sends the Corba messages to the replicated servers.
     *<dt>First it reads from a file the name of the machines where are running the replicated servers.
     *With the name of the machine, it can find the file where are written the IOR for the server which
     *is running on this machine.
     *<dt>It sends a dynamic message containing as argument the port and the address where it is listenning.
     *<dt>It doesn't wait for any response.
     */
    public void Announce()
    {
        Vector server =new Vector();


	java.util.Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "org.openorb.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass","org.openorb.CORBA.ORBSingleton");
	System.setProperties(props);

	org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init((java.lang.String[])null, props);



	String rep=null;
	try
	    {

		String  serverFile = uniClient.serverNameFile;
		FileInputStream servFile =new FileInputStream(serverFile);
		BufferedReader inServ= new BufferedReader(new InputStreamReader(servFile));
                int i =0;

		do
		    {
			rep=inServ.readLine();

                        if(rep!=null)
			    {

				int index=rep.lastIndexOf("");
				String rep1=rep.substring(0,index-2);
				String rep2=rep.substring(index-1);
				int pid = Integer.valueOf(rep2).intValue();

				java.net.InetAddress add=java.net.InetAddress.getByName(rep1);

				ProcessDetails proc = new ProcessDetails(pid,add,0);

				uniClient.receiverClient.processes.addElement(proc);

				server.addElement(rep1);



				i++;
			    }
		    }while(rep!=null);

		servFile.close();
	    }
	catch(IOException ex)
	    {

		System.err.println("Can't read from `" +
				   ex.getMessage() + "'");
		System.exit(1);
	    }




	try{


	    for(int i=0; i<server.size() ; i++)
		{

		    String ref = null;
                    boolean lecture=false;
		    try
			{
			    //String path="/home/x02isara/IOR/";

			    //String refFile = ((String)(server.elementAt(i))).concat("Application.ref");
			    //String pathNameFile=path.concat(refFile);
			    //java.io.File refF=new java.io.File(pathNameFile);

			     String refFile = ((String)(server.elementAt(i))).concat("Application.ref");

                            while(lecture==false)
				{
				    lecture=true;
				    try
					{
					    FileInputStream file = new FileInputStream(refFile);

					    BufferedReader in =
						new BufferedReader(new InputStreamReader(file));

					    ref = in.readLine();

					    file.close();
					    if((ref.compareTo(""))==0)
						{
						    lecture=false;
						}
					}
				    catch(FileNotFoundException fex)
					{
					    lecture=false;
					}
				}

			}
		    catch(IOException ex)
			{
			    System.err.println("Can't read from `" +
					       ex.getMessage() + "'");
			    System.exit(1);
			}



		    org.omg.CORBA.Object obj = orb.string_to_object(ref);
		    if(obj == null)
			throw new RuntimeException();
		    InetAddress address=uniClient.add;
                    String  add = address.getHostName();

		    NVList app_arguments=orb.create_list(3);
		    org.omg.CORBA.Any arg1=orb.create_any();
		    arg1.insert_long(uniClient.port);
		    app_arguments.add_value("param0",arg1,org.omg.CORBA.ARG_IN.value);
		    org.omg.CORBA.Any arg2=orb.create_any();
		    arg2.insert_string(add);
		    app_arguments.add_value("param1",arg2,org.omg.CORBA.ARG_IN.value);
		    org.omg.CORBA.Any arg3=orb.create_any();
		    arg3.insert_long(0);
		    app_arguments.add_value("param2",arg3,org.omg.CORBA.ARG_IN.value);


		    org.omg.CORBA.Request app_request = new org.openorb.CORBA.dii.Request(obj,"client",app_arguments,null,orb.create_environment(),null,orb.create_context_list(),orb);

		    app_request.send_oneway();






		}
	}
	catch(COMM_FAILURE ex)
	    {
		System.err.println(ex.getMessage());
		ex.printStackTrace();
		System.exit(1);
	    }
    }


}

