/**
  * Redistribution and use of this software and associated
  * documentation ("Software"), with or without modification, are
  * permitted provided that the following conditions are met:
  *
  * 1. Redistributions of source code must retain copyright statements
  *    and notices.  Redistributions must also contain a copy of this
  *    document.
  *
  * 2. Redistributions in binary form must reproduce the above
  *    copyright notice, this list of conditions and the following
  *    disclaimer in the documentation and/or other materials provided
  *    with the distribution.
  *
  * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
  *    products derived from this Software without prior written
  *    permission of one of the parties involved in the TRANSORG project. For
  * 	  written permission, please contact diasz@ida.liu.se.
  *
  * 4. Products derived from this Software may not be called "TRANSORG"
  *    nor may "TRANSORG" appear in their names without prior written
  *    permission.
  *
  * 5. Due credit should be given to the TRANSORG Project
  *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
  *
  * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
  * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  * PARTICULAR PURPOSE ARE DISCLAIMED.
  *
  * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
  * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
  * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
  * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
  * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
  *
  * $Id: NRServerInterceptor.java,v 1.0  diasz Exp $
  *
  * Date         Author  Changes
  */

package universal;
import java.util.*;
import java.io.*;
import java.net.*;
import universal.*;


/** Author		: Isabelle Ravot
  * Last changed: 25/07/02
  * Contact		: diasz@ida.liu.se
  */
/**
 *The server interceptor for the case when we don't replicate the server.
 */

public class NRServerInterceptor extends org.omg.CORBA.LocalObject implements org.omg.PortableInterceptor.ServerRequestInterceptor
{

    org.omg.PortableInterceptor.ORBInitInfo info;
    int ft_slot;
    String location;
    org.omg.CORBA.ORB orb1;
    org.openorb.CORBA.ORB orb;


    org.omg.IOP.CodecFactory factory;
    org.omg.IOP.Codec codec;


    int num=0;


    java.io.RandomAccessFile trace_file=null;

    public NRServerInterceptor(org.omg.PortableInterceptor.ORBInitInfo inf,int t_slot)
    {
	info = inf;
        ft_slot=t_slot;
	try{
	    location = java.net.InetAddress.getLocalHost().getHostName();
	}catch(java.net.UnknownHostException e){System.out.println("Error in req interc. constr.");e.printStackTrace(new java.io.PrintStream(System.out));location=null;}
	orb=((org.openorb.PI.OpenORBInitInfo)info).orb();

    }


    /**
     * get the transaction propagation context from the service context list and save it to the PICurrent
     * @param ri the client request
     */
    public  void receive_request_service_contexts(org.omg.PortableInterceptor.ServerRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {


	orb1=((org.openorb.PI.OpenORBInitInfo)info).orb();
	orb=(org.openorb.CORBA.ORB)orb1;


	//useful in order to make the consensus

	trace_file=(java.io.RandomAccessFile)orb.getFeature("trace_file");
	//we need the retention_id and the client_id in order to identify uniquely the request
	//object needed for the consensus

	//variable needed to store the reply


	//variable needed for the service context
	String client_id="";
	int retention_id=-1;
	long request_start_time=0;
	long expiration_time=0;
	org.omg.CORBA.Any any=null;
	org.omg.IOP.ServiceContext serviceCtx_req=null;
	boolean ex_req=true;
	universal.RequestServiceContext frctx;



	try
	    {
		factory = info.codec_factory();
		codec = factory.create_codec(new org.omg.IOP.Encoding(org.omg.IOP.ENCODING_CDR_ENCAPS.value, (byte)1, (byte)2));
	    }
	catch(java.lang.Exception e){e.printStackTrace(new java.io.PrintStream(System.out));return;}

	try
	    {
		serviceCtx_req = ri.get_request_service_context(universal.Constants.NUM_Service_Context);
	    }
	catch (org.omg.CORBA.BAD_PARAM bp)
	    {ex_req=false;}

	if(serviceCtx_req!=null&&ex_req){
	    try{
		any = codec.decode_value(serviceCtx_req.context_data, universal.RequestServiceContextHelper.type());
		ri.set_slot(ft_slot,any);
	    }catch(Exception e){System.out.println(location+" "+name()+" Error in req interc. cont.");e.printStackTrace(new java.io.PrintStream(System.out));return;}
	    frctx = universal.RequestServiceContextHelper.extract(any);
	    Debug.println("le client_id is :"+frctx.client_id);
	    Debug.println("THE RETENTION_ID is :"+frctx.retention_id);
	    Debug.println("le request_start_time is :"+frctx.request_start_time);
	    request_start_time=frctx.request_start_time;
	    client_id=frctx.client_id;
	    retention_id=frctx.retention_id;
	}

	long time=System.currentTimeMillis();
	universal.utils.Utils.trace(universal.utils.Utils.ENTER_RECEIVE_REQUEST,retention_id,client_id,time,ri.operation(),trace_file);

	Debug.println("Debug : we are in the interceptor for the "+ri.operation()+"The trace file is "+trace_file);

	/*	if(!methSgn[r].readMethod) {


		}*/

	universal.utils.Utils.trace(universal.utils.Utils.START_PROC,retention_id,client_id,time,ri.operation(),trace_file);
	System.out.println("************Start everything for request "+ri.operation()+" Retention id ## "+retention_id+" *************");
	//As we don't want to let the request go to the application, we need to compute it here.
	//So we need to have a reference to the application object


	universal.utils.Utils.trace(universal.utils.Utils.START_LOOKUP,retention_id,client_id,time,ri.operation(),trace_file);
	universal.utils.Utils.trace(universal.utils.Utils.STOP_LOOKUP,retention_id,client_id,time,ri.operation(),trace_file);
	universal.utils.Utils.trace(universal.utils.Utils.START_WAIT_UPD_OPER,retention_id,client_id,time,ri.operation(),trace_file);
	universal.utils.Utils.trace(universal.utils.Utils.STOP_WAIT_UPD_OPER,retention_id,client_id,time,ri.operation(),trace_file);
	universal.utils.Utils.trace(universal.utils.Utils.START_TENT_EXEC,retention_id,client_id,time,ri.operation(),trace_file);
	universal.utils.Utils.trace(universal.utils.Utils.START_GET_STATE_B,retention_id,client_id,time,ri.operation(),trace_file);
	universal.utils.Utils.trace(universal.utils.Utils.START_INVOKE,retention_id,client_id,time,ri.operation(),trace_file);
	/*	System.out.println("invoke 3");
	try
	    {
		result=method.invoke(appl,argu);
	    }
	catch(java.lang.reflect.InvocationTargetException e)
	    {

	    }

	    System.out.println("invoke 4");*/
    }






	public void receive_request(org.omg.PortableInterceptor.ServerRequestInfo ri)
		throws org.omg.PortableInterceptor.ForwardRequest
	{


		System.out.println("Receives a request for an object : " + ri.target_most_derived_interface() + ", operation : " + ri.operation() );
	}

    public void send_reply(org.omg.PortableInterceptor.ServerRequestInfo ri)
    {

	orb1=((org.openorb.PI.OpenORBInitInfo)info).orb();
	orb=(org.openorb.CORBA.ORB)orb1;


	//useful in order to make the consensus

	trace_file=(java.io.RandomAccessFile)orb.getFeature("trace_file");
	//we need the retention_id and the client_id in order to identify uniquely the request
	//object needed for the consensus

	//variable needed to store the reply


	//variable needed for the service context
	String client_id="";
	int retention_id=-1;
	long request_start_time=0;
	long expiration_time=0;
	org.omg.CORBA.Any any=null;
	org.omg.IOP.ServiceContext serviceCtx_req=null;
	boolean ex_req=true;
	universal.RequestServiceContext frctx;



	try
	    {
		factory = info.codec_factory();
		codec = factory.create_codec(new org.omg.IOP.Encoding(org.omg.IOP.ENCODING_CDR_ENCAPS.value, (byte)1, (byte)2));
	    }
	catch(java.lang.Exception e){e.printStackTrace(new java.io.PrintStream(System.out));return;}

	try
	    {
		serviceCtx_req = ri.get_request_service_context(universal.Constants.NUM_Service_Context);
	    }
	catch (org.omg.CORBA.BAD_PARAM bp)
	    {ex_req=false;}

	if(serviceCtx_req!=null&&ex_req){
	    try{
		any = codec.decode_value(serviceCtx_req.context_data, universal.RequestServiceContextHelper.type());
		ri.set_slot(ft_slot,any);
	    }catch(Exception e){System.out.println(location+" "+name()+" Error in req interc. cont.");e.printStackTrace(new java.io.PrintStream(System.out));return;}
	    frctx = universal.RequestServiceContextHelper.extract(any);
	    Debug.println("le client_id is :"+frctx.client_id);
	    Debug.println("THE RETENTION_ID is :"+frctx.retention_id);
	    Debug.println("le request_start_time is :"+frctx.request_start_time);
	    request_start_time=frctx.request_start_time;
	    client_id=frctx.client_id;
	    retention_id=frctx.retention_id;
	}

	System.out.println("Sends a reply for operation : " + ri.operation() );
	long time=System.currentTimeMillis();
	universal.utils.Utils.trace(universal.utils.Utils.START_GET_STATE_A,retention_id,client_id,time,ri.operation(),trace_file);

	universal.utils.Utils.trace(universal.utils.Utils.STOP_TENT_EXEC,retention_id,client_id,time,ri.operation(),trace_file);

	universal.utils.Utils.trace(universal.utils.Utils.START_CONSENSUS,retention_id,client_id,time,ri.operation(),trace_file);

	universal.utils.Utils.trace(universal.utils.Utils.STOP_CONSENSUS,retention_id,client_id,time,ri.operation(),trace_file);

	//throw an exception in order to avoid the request to go to the application
	System.out.println("#########finish for the request "+ri.operation()+" Retention id ## "+retention_id+" ############");

	universal.utils.Utils.trace(universal.utils.Utils.STOP_PROC_NORMAL,retention_id,client_id,time,ri.operation(),trace_file);

    }

	public void send_exception(org.omg.PortableInterceptor.ServerRequestInfo ri)
		throws org.omg.PortableInterceptor.ForwardRequest
	{}

    public void send_other(org.omg.PortableInterceptor.ServerRequestInfo ri)
    throws org.omg.PortableInterceptor.ForwardRequest
    {

    }
    public String name()
    {
	return "ServerInterceptor";
    }

  public void destroy() {
  }
}
