/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of one of the parties involved in the TRANSORG project. For
 * 	  written permission, please contact diasz@ida.liu.se.
 *
 * 4. Products derived from this Software may not be called "TRANSORG"
 *    nor may "TRANSORG" appear in their names without prior written
 *    permission.
 *
 * 5. Due credit should be given to the TRANSORG Project
 *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
 *
 * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
 *
 * $Id: ServerInterceptor.java,v 1.0   Exp $
 *
 */

package universal;
import java.util.*;
import java.io.*;
import java.net.*;
import universal.*;
import universal.leader.*;
import universal.communication.*;
import universal.messagein.*;
import universal.consensus.*;
/** Author		: Isabelle Ravot
* Date			: 23/04/02
* Last changed	: 30/05/02
* Contact		: diasz@ida.liu.se
*/


/**This is from this class that everything is going to be done (execution of the request, consensus, sending back of the reply).
 *<dt>This is done here because we want to be as much as possible transparent from the application that doesn't know
 *anything about consensus, leader election,...
 */
public class ServerInterceptor extends org.omg.CORBA.LocalObject implements org.omg.PortableInterceptor.ServerRequestInterceptor
{

    org.omg.PortableInterceptor.ORBInitInfo info;
    int ft_slot;
    String location;
    org.omg.CORBA.ORB orb1;
    org.openorb.CORBA.ORB orb;
    /**This object permit to access the implementation of the algorithm (Leader election, consensus)*/
    Universal uni;
    org.omg.IOP.CodecFactory factory;
    org.omg.IOP.Codec codec;

    /**Num is the idea for this server of the next position in the total order, the next request should be.*/
    int num=0;
    /**
     *The variable store contains the outcome (Request, Reply, Update) for requests that have
     *already been processed and that have not yet expired.
     *<dt>It is useful in case of the same request is sent twice but we don't want to execute it twice.
     */
    OutcomeStore store=new OutcomeStore();
    /**
     *Contains the description of all the methods of the application server.
     *<dt>The description is the name, the modes (IN, INOUT, OUT) and types of the parameters.
     */
    universal.MethTable table;
    universal.MethSignStr[] methSgn;
    /**Permit to synchronized on this object in order to treat each request one after another*/
    Object one_by_one=new Object();
    /**Number of request that wait to be processed*/
    int number_waiting=0;
    int number_notify=0;
    /**The file which contains the traces.*/
    java.io.RandomAccessFile trace_file=null;

    public ServerInterceptor(org.omg.PortableInterceptor.ORBInitInfo inf,int t_slot)
    {
	info = inf;
        ft_slot=t_slot;
	try{
	    location = java.net.InetAddress.getLocalHost().getHostName();
	}catch(java.net.UnknownHostException e){System.out.println("Error in req interc. constr.");e.printStackTrace(new java.io.PrintStream(System.out));location=null;}
	orb=((org.openorb.PI.OpenORBInitInfo)info).orb();

    }


    /**
     *This is precisely from that method that everything is going to be done.
     *<dt>First checks if the request, it has received, is a message from one of the other replicated servers or
     *from the client to inform it in which port and on which IP address they are listenning.
     *<dt> If it is the case the name of the operation is either replicas or client.
     *<dt>We take the informations (port, address) and register them (create the object ProcessDetails).
     *<dt>If it is a request from the client which wants to execute one method of the application server,
     *we do the following steps:
     *<dt>- extracts the service context (retention_id, client_id, request_start_time)
     *<dt>- check if the request has already been processed(OutcomeStore). If it the case, the reply
     *is directly sent back to the client.
     *<dt>- Otherwise the request waits until all the request that has arrived before are processed.
     *<dt>- Check again if during the time the request was waiting, it hasn't already been executed.
     *<dt>- Get the state before the execution
     *<dt>- Execute the method.
     *<dt>- Get the update.
     *<dt>- Check if the server trusts itself as the leader
     *<dt>- If it doesn't, the state as it was before the execution is restored and an exception is
     *sent to the client to inform it that it should resend the request.
     *<dt>- If it was the leader, the consensus is made.
     *<dt>- If the result of the consensus was not the request, it has proposed. It restart from the beginning with
     *the same request but for the next free position.
     *<dt>-If it was the request it has proposed, it sent back the reply to the leader and throw an exception
     *to avoid the request to go until the application.
     * @param ri the client request
     */
    public  void receive_request_service_contexts(org.omg.PortableInterceptor.ServerRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest,org.omg.CORBA.STOP_REQUESTException
    {


	orb1=((org.openorb.PI.OpenORBInitInfo)info).orb();
	orb=(org.openorb.CORBA.ORB)orb1;

	uni = (Universal)(orb.getFeature("universal"));

	if((((ri.operation()).compareTo("replicas"))==0)||(((ri.operation()).compareTo("client"))==0))
	    {


		try
		    {

			org.omg.CORBA.portable.InputStream in=((org.openorb.net.ServerRequest)ri).argument_stream();
			int port = in.read_long();
			String address= in.read_string();
			int pid =in.read_long();
			InetAddress add =java.net.InetAddress.getByName(address);


			ProcessDetails proc = new ProcessDetails(pid,add,port);

			if(((ri.operation()).compareTo("replicas"))==0)
			    {
				//Debug.println("Receive message I am in from :"+add);

				uni.sender.processesReplicas.addElement(proc);


			    }
			Debug.println("Receive message from :"+add);
			uni.sender.processes.addElement(proc);


		    }
		catch(java.net.UnknownHostException e)
		    {
			System.out.println("Error in req interc. constr.");
			e.printStackTrace(new java.io.PrintStream(System.out));
		    }


		throw new org.omg.CORBA.STOP_REQUESTException();
	    }
	//useful in order to make the consensus
	else
	    {
		trace_file=(java.io.RandomAccessFile)orb.getFeature("trace_file");
		//we need the retention_id and the client_id in order to identify uniquely the request
		//object needed for the consensus

		//variable needed to store the reply
		Outcome prop=null;
		Decision decision=new Decision();

		Reply repl;
		Update upd=null;
		Request req;


		//variable needed for the service context
		String client_id="";
		int retention_id=-1;
		long request_start_time=0;
		long expiration_time=0;
		org.omg.CORBA.Any any=null;
		org.omg.IOP.ServiceContext serviceCtx_req=null;
		boolean ex_req=true,first=true;
		universal.RequestServiceContext frctx;

		StoreResult sResult=new StoreResult(false,null);
		java.lang.reflect.Method methods[],method=null,getState=null,setState=null,set_update=null,get_update=null;
	       	java.lang.Object state[]=new java.lang.Object[1];
		byte[]	beforeState=null;
		//Byte[]	beforeStateb=null;
		java.lang.Object  beforeStateByte=null;
		byte[] afterState=null;
		java.lang.Object  afterStateByte=null;
		byte[] update=null;
		java.lang.Object  updateByte=null;

		//variable needed for sending back the reply
		org.openorb.iiop.CDRInputStream is;
		org.omg.Dynamic.Parameter[] param;
		java.lang.Object argu[]=null;
		int nr_of_out_inout=0;

		java.lang.Object result=null;
		java.lang.Object[] result_arg=null;
		org.omg.CORBA.Any[] ret_args=null;
		org.omg.CORBA.Any reply=null;


		try
		    {
			factory = info.codec_factory();
			codec = factory.create_codec(new org.omg.IOP.Encoding(org.omg.IOP.ENCODING_CDR_ENCAPS.value, (byte)1, (byte)2));
		    }
		catch(java.lang.Exception e){e.printStackTrace(new java.io.PrintStream(System.out));return;}

		try
		    {
			serviceCtx_req = ri.get_request_service_context(universal.Constants.NUM_Service_Context);
		    }
		catch (org.omg.CORBA.BAD_PARAM bp)
		    {ex_req=false;}

		if(serviceCtx_req!=null&&ex_req){
		    try{
			any = codec.decode_value(serviceCtx_req.context_data, universal.RequestServiceContextHelper.type());
			ri.set_slot(ft_slot,any);
		    }catch(Exception e){System.out.println(location+" "+name()+" Error in req interc. cont.");e.printStackTrace(new java.io.PrintStream(System.out));return;}
		    frctx = universal.RequestServiceContextHelper.extract(any);
		    Debug.println("le client_id is :"+frctx.client_id);
		    Debug.println("THE RETENTION_ID is :"+frctx.retention_id);
		    Debug.println("le request_start_time is :"+frctx.request_start_time);
		    request_start_time=frctx.request_start_time;
		    client_id=frctx.client_id;
		    retention_id=frctx.retention_id;
		}


		universal.utils.Utils.trace(universal.utils.Utils.ENTER_RECEIVE_REQUEST,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);

		Debug.println("Debug : we are in the interceptor for the "+ri.operation()+"The trace file is "+trace_file);

		table=(universal.MethTable)orb.getFeature("methsign");

		if(table==null) {System.out.println("No method table");return;}
		methSgn=table.methods;

		is=(org.openorb.iiop.CDRInputStream)((org.openorb.net.ServerRequest)ri).argument_stream();

		int r;
		boolean found=false;
		for(r=0;r<methSgn.length;r++)
		    {

			if(methSgn[r].name.equals(ri.operation())){
			    found=true;
			    break;
			}
		    }

		if(!found) {System.out.println("Too bad!!! Method not found!!"); return;}
		Debug.println("Before wait request "+ri.operation()+" Retention id ## "+retention_id);




		universal.utils.Utils.trace(universal.utils.Utils.START_PROC,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);

		//As we don't want to let the request go to the application, we need to compute it here.
		//So we need to have a reference to the application object

		java.lang.Object appl=orb.getFeature("application");
                expiration_time=request_start_time+universal.Constants.EXPIRATION_TIME;
                req=new Request(retention_id,client_id,expiration_time);
		while(true)
		    {
			if(!methSgn[r].readMethod) {
			    universal.utils.Utils.trace(universal.utils.Utils.START_LOOKUP,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
			    Debug.println("Before committed");
			    sResult=store.isCommitted(req);
			    Debug.println("After committed");
			    universal.utils.Utils.trace(universal.utils.Utils.STOP_LOOKUP,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
			}
			if(sResult.bool)
			    {
				System.out.println("resend1");
				result=sResult.out.res.res;
				System.out.println("resend2");
				result_arg=sResult.out.res.res_args;
				System.out.println("resend3");
				if(result_arg==null)
				    {
					System.out.println("ret arg null");
					ret_args=new org.omg.CORBA.Any[0];

				    }
				else
				    {
					System.out.println("ret arg not null");
					ret_args=new org.omg.CORBA.Any[result_arg.length];

				System.out.println("resend4");
				//Debug.println(result_arg.length);
				for(int k=0;k<result_arg.length;k++)
				    {
					ret_args[k]=universal.utils.Utils.give_any_arg(orb,result_arg[k]);
					if(ret_args[k]==null) {break;}
				    }
				    }
				Debug.println("The request has already been executed "+result);

				reply=universal.utils.Utils.give_any(orb,result,false);
				int tkvr=-1,tkvrc=Integer.MIN_VALUE,i;
				boolean error=false;
				if(ri.response_expected()){


				    org.omg.CORBA.portable.OutputStream os;
				    os=((org.openorb.net.ServerRequest)ri).createNoInvokeReply();
				    tkvr=reply.type().kind().value();
				    if(tkvr==org.omg.CORBA.TCKind._tk_TypeCode)
					tkvrc=reply.extract_TypeCode().kind().value();
				    if(tkvr!=org.omg.CORBA.TCKind._tk_TypeCode||tkvrc!=org.omg.CORBA.TCKind._tk_void)
					universal.utils.Utils.write_output_stream(reply,os);
				    for(int j=0;j<ret_args.length;j++)
					universal.utils.Utils.write_output_stream(ret_args[j],os);


				    //throw an exception in order to avoid the request to go to the application

				    //Debug.println("The number of threads still waiting"+number_waiting);

				}

				universal.utils.Utils.trace(universal.utils.Utils.STOP_PROC_REPLY_THERE,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
				throw new org.omg.CORBA.STOP_REQUESTException();

			    }
			else
			    {
				if(first){
				    first=false;
				    if(!methSgn[r].readMethod) {
					synchronized(one_by_one)
					    {

						int local_number_waiting=number_waiting;
						number_waiting++;
						universal.utils.Utils.trace(universal.utils.Utils.START_WAIT_UPD_OPER,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
						//System.out.println("Before wait the number of block process "+local_number_waiting+" request "+ri.operation()+" Retention id ## "+retention_id);
						while(local_number_waiting>0)
						    {
							int number_notify_bf=number_notify;
							//System.out.println("The number notify before "+number_notify_bf+" request "+ri.operation()+" Retention id ## "+retention_id);
							try


							    {

								one_by_one.wait();

							    }
							catch(java.lang.InterruptedException e)
							    {
								e.printStackTrace();
							    }
							int number_notify_af=number_notify;
							//System.out.println("The number notify after "+number_notify_af+" request "+ri.operation()+" Retention id ## "+retention_id);
							local_number_waiting=local_number_waiting-(number_notify_af-number_notify_bf);
							//System.out.println("one process as been deblock sill waiting "+local_number_waiting+" request "+ri.operation()+" Retention id ## "+retention_id);
						    }
						universal.utils.Utils.trace(universal.utils.Utils.STOP_WAIT_UPD_OPER,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
					    }
				    }
				    if(!methSgn[r].readMethod)
					{
					    universal.utils.Utils.trace(universal.utils.Utils.START_LOOKUP,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
					    Debug.println("Before committed");
					    sResult=store.isCommitted(req);
					    Debug.println("After committed");
						universal.utils.Utils.trace(universal.utils.Utils.STOP_LOOKUP,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
					}
				    if(sResult.bool)
					{
					    result=sResult.out.res.res;
					    result_arg=sResult.out.res.res_args;
					    if(result_arg==null)
						{
						    ret_args=new org.omg.CORBA.Any[0];

						    Debug.println("null");
						}
					    else
						{
						    Debug.println("not null "+result_arg.length);
						    ret_args=new org.omg.CORBA.Any[result_arg.length];

				//Debug.println(result_arg.length);
						    for(int k=0;k<result_arg.length;k++)
							{
							    ret_args[k]=universal.utils.Utils.give_any_arg(orb,result_arg[k]);
							    if(ret_args[k]==null) {break;}
							}
						}
					    Debug.println("The request "+retention_id+" has already been executed "+result);

					    reply=universal.utils.Utils.give_any(orb,result,false);
					    int tkvr=-1,tkvrc=Integer.MIN_VALUE,i;
					    boolean error=false;
					    if(ri.response_expected()){


						org.omg.CORBA.portable.OutputStream os;
						os=((org.openorb.net.ServerRequest)ri).createNoInvokeReply();
						tkvr=reply.type().kind().value();
						if(tkvr==org.omg.CORBA.TCKind._tk_TypeCode)
						    tkvrc=reply.extract_TypeCode().kind().value();
						if(tkvr!=org.omg.CORBA.TCKind._tk_TypeCode||tkvrc!=org.omg.CORBA.TCKind._tk_void)
						    universal.utils.Utils.write_output_stream(reply,os);
						for(int j=0;j<ret_args.length;j++)
						    universal.utils.Utils.write_output_stream(ret_args[j],os);


						//throw an exception in order to avoid the request to go to the application

						//Debug.println("The number of threads still waiting"+number_waiting);

					    }
					    synchronized(one_by_one)
						{
						    number_notify=number_notify+1;
						    //System.out.println("The number of notify  "+number_notify+" for the request "+ri.operation()+" for the retention_id "+retention_id);
						    number_waiting=number_waiting-1;

						    one_by_one.notifyAll();

						}
					    universal.utils.Utils.trace(universal.utils.Utils.STOP_PROC_REPLY_THERE,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
					    throw new org.omg.CORBA.STOP_REQUESTException();
					}
				    Debug.println("************Start everything for request "+ri.operation()+" Retention id ## "+retention_id+" *************");
				    param=universal.utils.Utils.read_args(orb,methSgn[r],is);
				    argu=new java.lang.Object[param.length];

				    for(int k=0;k<param.length;k++)
					{
					    if(param[k].mode.value()!=org.omg.CORBA.ParameterMode._PARAM_IN){
						argu[k]=universal.utils.Utils.give_object_holder(param[k].argument,param[k].mode.value()==org.omg.CORBA.ParameterMode._PARAM_OUT);
						nr_of_out_inout++;
					    }
					    else
						argu[k]=universal.utils.Utils.give_object(param[k].argument);
					}
				}
				//org.omg.CORBA.Any reply=null;
				//org.omg.CORBA.Any[] ret_args=null;
				try
				    {
					universal.utils.Utils.trace(universal.utils.Utils.START_TENT_EXEC,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
					//java.lang.Object result=null;
					//java.lang.reflect.Method methods[],method=null,getState=null,setState=null;
					methods=appl.getClass().getMethods();
					for(int j=0;j<methods.length;j++)
					    if(methods[j].getName().equals(ri.operation())){
						method=methods[j];
						Debug.println("Found the method for oper "+ri.operation());
						break;
					    }

					if(!methSgn[r].readMethod) {

					    for(int j=0;j<methods.length;j++)
						if(methods[j].getName().equals("get_state")){
						    getState=methods[j];
						    Debug.println("found getState");
						    break;
						}

					    for(int j=0;j<methods.length;j++)
						if(methods[j].getName().equals("set_state")){
						    setState=methods[j];
						    break;
						}

					    for(int j=0;j<methods.length;j++)
						if(methods[j].getName().equals("get_update")){
						    get_update=methods[j];
						    Debug.println("found get_update");
						    break;
						}

					    for(int j=0;j<methods.length;j++)
						if(methods[j].getName().equals("set_update")){
						    set_update=methods[j];
						    break;
						}



					//String rep;




					    universal.utils.Utils.trace(universal.utils.Utils.START_GET_STATE_B,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
					    Debug.println("invoke 1");
					    beforeStateByte=getState.invoke(appl,null);
					    Debug.println("invoke 2");

					    beforeState=(byte[])beforeStateByte;
					}
					universal.utils.Utils.trace(universal.utils.Utils.START_INVOKE,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
					Debug.println("invoke 3");
					try
					    {
						result=method.invoke(appl,argu);
					    }
					catch(java.lang.reflect.InvocationTargetException e)
					    {

					    }
					if(!methSgn[r].readMethod) {
					    Debug.println("invoke 4");
					    universal.utils.Utils.trace(universal.utils.Utils.START_GET_STATE_A,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
					    Debug.println("invoke 5");
					    updateByte=get_update.invoke(appl,null);
					    Debug.println("invoke 6");
					    //afterStateByte=getState.invoke(appl,null);
					    universal.utils.Utils.trace(universal.utils.Utils.STOP_TENT_EXEC,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
					    update=(byte[])updateByte;
					    //afterState=(byte[])afterStateByte;
					    //Debug.println("The number of out and inout "+nr_of_out_inout);
					}
					ret_args=new org.omg.CORBA.Any[nr_of_out_inout];
					result_arg= new java.lang.Object[nr_of_out_inout];
					reply=universal.utils.Utils.give_any(orb,result,false);

					int f=0;
					for(int k=0;k<argu.length;k++)
					    {
						int i_o=argu[k].getClass().getName().indexOf("Holder");
						if(i_o!=-1&&argu[k].getClass().getName().substring(i_o).equals("Holder")){
						    ret_args[f]=universal.utils.Utils.give_any(orb,argu[k],true);
						    try
							{
							  result_arg[f]=argu[k].getClass().getField("value").get(argu[k]);
							}
						    catch(java.lang.Exception e)
							{
							    e.printStackTrace();
							}
						    if(ret_args[f]==null) {break;}
						    f++;
						}
					    }
					if(!methSgn[r].readMethod) {
					    repl=new Reply(result,result_arg);
					    upd=new Update(update);
					    //upd=new Update(afterState);
					    prop=new Outcome(req,repl,upd);
					    //Debug.println("The method sign is "+methSgn[r].readMethod);
					    //(uni.cons).add(new Consensus(uni,num));
					    int roundno=uni.sender.pid;
					    uni.cons =new Consensus(uni,num);
					    decision.deci=false;

					    while((decision.deci==false)&&(uni.lead.is_leader()==uni.sender.pid))
						{
						    universal.utils.Utils.trace(universal.utils.Utils.START_CONSENSUS,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
						    decision = uni.cons.propose(roundno, prop);
						    roundno=roundno+uni.sender.totalno();
						    universal.utils.Utils.trace(universal.utils.Utils.STOP_CONSENSUS,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
						}

					    if (decision.deci==false)
						{
						    universal.utils.Utils.trace(universal.utils.Utils.START_RESTORE_ST,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
						    //(uni.cons).remove(uni.cons.size()-1);
						    Debug.println("Decision was false");
						    state[0]=(java.lang.Object)beforeState;
						    setState.invoke(appl,state);
						    Debug.println("We have restore the state");
						    universal.utils.Utils.trace(universal.utils.Utils.STOP_RESTORE_ST,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
						    synchronized(one_by_one)
							{
							    number_notify=number_notify+1;
							    //System.out.println("The number of notify  "+number_notify+" for the request "+ri.operation()+" for the retention_id "+retention_id);
							    number_waiting=number_waiting-1;

							    one_by_one.notifyAll();

							}
						    /*	org.omg.CORBA.Any exception =orb.create_any();
							org.omg.CORBA.COMM_FAILUREHelper.insert(exception,new org.omg.CORBA.COMM_FAILURE());*/
						    org.omg.CORBA.portable.OutputStream os;
						    os=((org.openorb.net.ServerRequest)ri).createExceptionReply();
						    universal.exception.NotUserExceptionHelper.write(os,new universal.exception.NotUserException());
						    //	universal.utils.Utils.write_output_stream(exception,os);
						    universal.utils.Utils.trace(universal.utils.Utils.STOP_PROC_EXC,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
						    throw new org.omg.CORBA.STOP_REQUESTException();
						}

					    num=num+1;
					    store.setCommitted(decision.out);
					}
					else
					    {
						/* if it was only a read method the server executes
						 * the request without making the consensus
						 */
						decision.deci=true;
						decision.out=prop;
					    }


				    }

				catch(java.lang.IllegalAccessException e)
				    {
					System.out.println("IllegalAccessException ");
					e.printStackTrace(new java.io.PrintStream(System.out));
				    }
				catch(java.lang.reflect.InvocationTargetException e)
				    {
					System.out.println("InvocationTargetException ");
					e.printStackTrace(new java.io.PrintStream(System.out));
				    }

				/*

				  ((application.Application_impl)appl).setState(decision.out.upd.upd);
				*/
				/*
				  try
				  {
				  universal.utils.Utils.trace(universal.utils.Utils.SET_STATE,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
				  state[0]=(java.lang.Object)beforeState;
				  setState.invoke(appl,state);
				  universal.utils.Utils.trace(universal.utils.Utils.SET_UPDATE,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);

				  state[0]=(java.lang.Object)upd.upd;
				  set_update.invoke(appl,state);
				  universal.utils.Utils.trace(universal.utils.Utils.STOP_SET_UPDATE,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
				  }
				  catch(java.lang.IllegalAccessException e)
				  {
				  System.out.println("IllegalAccessException ");
				  e.printStackTrace(new java.io.PrintStream(System.out));
				  }
				  catch(java.lang.reflect.InvocationTargetException e)
				  {
				  System.out.println("InvocationTargetException ");
				  e.printStackTrace(new java.io.PrintStream(System.out));
				  }
				*/

				if(decision.out!=prop)
				    {

					try
					    {

						/*
						  state[0]=(java.lang.Object)decision.out.upd.upd;
						  setState.invoke(appl,state);
						*/

						universal.utils.Utils.trace(universal.utils.Utils.SET_STATE,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);
						state[0]=(java.lang.Object)beforeState;
						setState.invoke(appl,state);
						universal.utils.Utils.trace(universal.utils.Utils.SET_UPDATE,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);

						state[0]=(java.lang.Object)decision.out.upd.upd;
						set_update.invoke(appl,state);
						universal.utils.Utils.trace(universal.utils.Utils.STOP_SET_UPDATE,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);


					    }
					catch(java.lang.IllegalAccessException e)
					    {
						System.out.println("IllegalAccessException ");
						e.printStackTrace(new java.io.PrintStream(System.out));
					    }
					catch(java.lang.reflect.InvocationTargetException e)
					    {
						System.out.println("InvocationTargetException ");
						e.printStackTrace(new java.io.PrintStream(System.out));
					    }


				    }
				else
				    {

					//org.omg.CORBA.Any reply=orb.create_any();
					int tkvr=-1,tkvrc=Integer.MIN_VALUE,i;
					boolean error=false;
					if(ri.response_expected()){
					    //  reply.insert_Object((org.omg.CORBA.Object)result);

					    org.omg.CORBA.portable.OutputStream os;
					    os=((org.openorb.net.ServerRequest)ri).createNoInvokeReply();
					    tkvr=reply.type().kind().value();
					    if(tkvr==org.omg.CORBA.TCKind._tk_TypeCode)
						tkvrc=reply.extract_TypeCode().kind().value();
					    if(tkvr!=org.omg.CORBA.TCKind._tk_TypeCode||tkvrc!=org.omg.CORBA.TCKind._tk_void)
						universal.utils.Utils.write_output_stream(reply,os);
					    // Debug.println("number of return argumetn is "+ret_args.length);
					    for(int j=0;j<ret_args.length;j++)
						universal.utils.Utils.write_output_stream(ret_args[j],os);


					}

				        //throw an exception in order to avoid the request to go to the application
					System.out.println("#########finish for the request "+ri.operation()+" Retention id ## "+retention_id+" ############");
					synchronized(one_by_one)
					    {
						number_notify=number_notify+1;
						//System.out.println("The number of notify  "+number_notify+" for the request "+ri.operation()+" for the retention_id "+retention_id);
						number_waiting=number_waiting-1;

						one_by_one.notifyAll();

					    }
					universal.utils.Utils.trace(universal.utils.Utils.STOP_PROC_NORMAL,retention_id,client_id,System.currentTimeMillis(),ri.operation(),trace_file);

					throw new org.omg.CORBA.STOP_REQUESTException();
				    }
			    }
		    }
		}



    }


	public void receive_request(org.omg.PortableInterceptor.ServerRequestInfo ri)
		throws org.omg.PortableInterceptor.ForwardRequest
	{


		System.out.println("Receives a request for an object : " + ri.target_most_derived_interface() + ", operation : " + ri.operation() );
	}

	public void send_reply(org.omg.PortableInterceptor.ServerRequestInfo ri)
	{

		System.out.println("Sends a reply for operation : " + ri.operation() );
	}

	public void send_exception(org.omg.PortableInterceptor.ServerRequestInfo ri)
		throws org.omg.PortableInterceptor.ForwardRequest
	{}

    public void send_other(org.omg.PortableInterceptor.ServerRequestInfo ri)
    throws org.omg.PortableInterceptor.ForwardRequest
    {

    }
    public String name()
    {
	return "ServerInterceptor";
    }

  public void destroy() {
  }
}
